/**
 * @preserve
 * JS Scripts for Bulk Table Editor for WooCommerce
 * by Consortia AS
 * @package BulkTableEditor/js
 * @endpreserve
 */

(function($) {

const loc_url         = new URL(window.location.href);
const view            = loc_url.searchParams.get('view');
var tbl               = '#wbtetable';
var tbl_ext           = '#wbtetable_ext';
var tbl_custom        = '#wbtetable_custom';
var tbl_cat           = '#wbtetable_cat';
var $                 = jQuery;
var aDefault          = '';
var aColor            = $('a').css('color');
var sum_col           = 8;
var custom_col_active = wbte_object.js_custom_active;
var page_view         = view ? view : 'prod';
var show_sku_home     = wbte_object.js_show_sku;
var vendor_active     = wbte_object.js_show_vendors;
var cur_tbl           = ( 'ext' === page_view ) ? tbl_ext : tbl;
var tbl_sale_filter   = wbte_object.js_sale_filter;
var custom_col        = '';

switch( page_view ) {
	case 'ext':
		cur_tbl = tbl_ext;
		break;
	case 'custom':
		cur_tbl = tbl_custom;
		break;
	case 'cat':
		cur_tbl = tbl_cat;
		break;
	default:
		cur_tbl = tbl;
		break;
}

if ( 'yes' === custom_col_active ) {
	custom_col = ', 8';
	sum_col++;
}

if ( 'yes' === vendor_active ) {
	sum_col++;
}

if ( 'yes' === wbte_object.brand_active ) {
	sum_col++;
}

window.addEventListener('loading', wbteProgressBar('start'));

jQuery(document).ready(function ( $ ) {

	$( document ).tooltip();

	if (cas_getItemWithExpiration('row_search')) {
		$('#product_search').val(cas_getItemWithExpiration('row_search'));
	}

	if ( 'ext' === page_view ) {
		$( tbl_ext ).tableCalc({
			calcColumns: [ 6, 7, 8, 9 ], 
			calcColumn_sum: 12,
			rowId: 0, 
			textColumns: [ 1, 2, 3, 4, 5 ],
			calcType: 'c', 
			calcCustom: '',
			decimals: 2, 
			calcOnLoad: false, 
			onEvent: '', 
		});

		$( tbl_ext +' tr td input' ).on( 'change', extChanged );
		$( tbl_ext +' tr td select' ).on( 'change', extChanged );
		

	} else if ( 'prod' === page_view ) {
		$( tbl ).tableCalc({
			calcColumns: [ 2, 3, 4, 7 + custom_col ], 
			calcColumn_sum: sum_col,
			rowId: 0, 
			textColumns: [ 5, 6 ], 
			calcType: 'c', 
			calcCustom: '(0:0 * 0:1)',
			decimals: 2, 
			calcOnLoad: false, 
			onEvent: 'change', 
		});

		$( tbl +' tr td input' ).on( 'change', calcChanged );
		$( tbl +' tr td select' ).on( 'change', calcChanged );

		calcChanged( true );

		//Range filter
		casRangeTypeChanged('','', false);//use false on load

		//Brands
		if ( 'yes' === wbte_object.brand_active ) {
			$('#product_brand_select').attr('multiple','multiple').multiselect({
				noneSelectedText: 'Select Brands',
				selectedList: 4,
				buttonWidth: 170,
			});
		}

	} else if ( 'upsell' === page_view ) { // upsell
		
		
		$('#wbte-category-filter').attr('multiple','multiple').multiselect({
			noneSelectedText: 'Select Categories',
			selectedList: 4,
			menuHeight: 400,
			buttonWidth: 250,
		}).multiselectfilter(
			{
				width:160,
			}
		);
		
		$('#wbte-category-filter-upsell').attr('multiple','multiple').multiselect({
			noneSelectedText: 'Select Categories',
			selectedList: 4,
			menuHeight: 400,
			buttonWidth: 250,
		}).multiselectfilter(
			{
				width:160,
			}
		);

		var sel_cats = $("#cas-form-category-filter input[name='categories']").val();
		if ( sel_cats.length > 0 ) {
			sel_cats = sel_cats.split(',');
			
			$("#wbte-category-filter option").each(function() {
				if ( '' === $(this).val() ) {
					$(this).prop('selected', false);
				}
				if ($.inArray($(this).val(), sel_cats) !== -1) {
					$(this).prop('selected', true);
				}
			});
			
			//Refesh multiselect
			$("#wbte-category-filter").multiselect("refresh");
			
		}

		var sel_cats2 = $("#cas-form-category2-filter input[name='categories2']").val();
		if ( sel_cats2.length > 0 ) {
			sel_cats2 = sel_cats2.split(',');
			
			$("#wbte-category-filter-upsell option").each(function() {
				if ( '' === $(this).val() ) {
					$(this).prop('selected', false);
				}
				if ($.inArray($(this).val(), sel_cats2) !== -1) {
					$(this).prop('selected', true);
				}
			});
			
			//Refesh multiselect
			$("#wbte-category-filter-upsell").multiselect("refresh");
			
		}

		$("#wbte-category-filter").on('change', function() {
			var selectedCategories = $(this).val(); 
		});

		$('#cas-on-off-switch-div').on('click', function() {
			var is_checked = jQuery('#cas-search-toggle').prop('checked');
			$('#cas-search-toggle').prop('checked', (is_checked) ? '' : 'checked');
		});

		$('#cas-on-off-switch-div2').on('click', function() {
			var is_checked = jQuery('#cas-search-toggle2').prop('checked');
			$('#cas-search-toggle2').prop('checked', (is_checked) ? '' : 'checked');
		});

		//Click events for tables
		$('#wbte-linked-product-table tbody tr').on('click', function(event){
			var input = $(this).find('input[name="productId"]');
			if (event.target !== input[0]) {
				input.prop('checked', !input.prop('checked'));
			}
			$(this).toggleClass('cas-selected-row', input.prop('checked'));
		});

		$('#wbte-up-cross-sell-table tbody tr').on('click', function(event){
			var input = $(this).find('input[name="productUp"]');
			if (event.target !== input[0]) {
				input.prop('checked', !input.prop('checked'));
			}
			$(this).toggleClass('cas-selected-row', input.prop('checked'));
		});

		//Row search
		$( '#wbte_s' ).on( 'keypress', function(event) {
			if (event.key === 'Enter'){
				wbte_SearchSubmit($('#wbte_upsell_search_filter'));
			} 
		});
		$( '#wbte_s' ).on( 'input', function() {
			if ( this.value === '' ){
				$('#wbte-linked-product-table tbody tr').show();
			}
		});
		$( '#wbte_s2' ).on( 'keypress', function(event) {
			if (event.key === 'Enter'){
				wbte_SearchSubmit2($('#wbte_upsell_search_filter2'));
			} 
		});
		$( '#wbte_s2' ).on( 'input', function() {
			if ( this.value === '' ){
				$('#wbte-up-cross-sell-table tbody tr').show();
			}
		});
	} else if ( 'custom' === page_view ) {
		
		$('#bulk_add_attribute').attr('multiple','multiple').multiselect({
			noneSelectedText: 'Select Attributes',
			selectedList: 4,
			buttonWidth: 250,
			menuHeight: 400,
			optgroupHeader: true,
		}).multiselectfilter(
			{
				width:180,
			}
		);

		$("#bulk_add_attribute option").each(function() {
			if ( '' === $(this).val() ) {
				$(this).prop('selected', false);
			}
		});

		$("#bulk_add_attribute").multiselect("refresh");

		$( tbl_custom +' tr td input' ).on( 'change', extChanged );
	}

	//Home and Other values
	if ( 'upsell' !== page_view ) {

		if ( 'cat' === page_view ) {

			//Create the progressbar
			$( "#wbte_progressbar" ).progressbar({
				value: 0
			}).hide();

			//Loop and set gallery images
			$('#wbte_img_gallery').on('bulkSetEvent', function() {
				wbteBulkSetRowImages();
			});
			/*
			// All AJAX requests are done, hide the progress bar
			$(document).ajaxStop(function() {
				$("#wbte_progressbar").hide();
			});*/

			//Multiselect Bulk Category Selector
			$('#bulk_product_cat').attr('multiple','multiple').multiselect({
				noneSelectedText: 'Select Categories',
				selectedList: 4,
				menuHeight: 400,
				buttonWidth: 250,
			}).multiselectfilter(
				{
					width:160,
				}
			);

			//Single categories to select
			$('#wbte_select_single_categories_div').dialog({
				autoOpen: false,
				width: 400,
			});

			//Open dialog
			$('.wbte-edit-single-cat').on('click', function() {
				var row        = $(this).closest('tr'); 
				var editButton = $(this);

				row.css('background','lightgray');

				// Store the row in the dialog's data for later use
				$('#wbte_select_single_categories_div').data('selectedRow', row);

				// Open the dialog and set its position relative to the editButton
				$('#wbte_select_single_categories_div').dialog("option", "position", {
					my: "right+34 top", // Dialog's corner
					at: "left top+18", // Target's (editButton's) corner
					of: editButton, 
					collision: "fit"
				}).dialog("open");
			
			});

			//Multiselect for single categories
			$('#wbte_select_single_categories').attr('multiple','multiple').multiselect({
				noneSelectedText: 'Select Categories',
				selectedList: 4,
				menuHeight: 400,
				buttonWidth: 370,
			}).multiselectfilter(
				{
					width:300,
				}
			);

			$('#bulkcat_select_single_type').on('change', function() {
				var selectedRow       = $('#wbte_select_single_categories_div').data('selectedRow'); // Retrieve the stored row
				var selectType        = $('#bulkcat_select_single_type option:selected').val();
				var form_elements     = selectedRow[0].children[0].children[0];
				var category_arr      = [];
				var category_arr_full = [];
				
				$('#wbte_select_single_categories option').each(function(){
					if ( '' !== this.value ) {
						if ( this.selected ) {
							category_arr.push( { id: this.value, name: this.text.trim() } );
						}
						category_arr_full.push( { id: this.value, name: this.text.trim() } );
					}
				});
				wbte_setCategory( selectType,form_elements, category_arr, category_arr_full );
				//Reset and close
				$('#wbte_select_single_categories').multiselect('uncheckAll');
				$('#wbte_select_single_categories').multiselect('refresh');
				$('#bulkcat_select_single_type').val('0');
				$( form_elements['id'] ).prop( 'checked', true );
				$('#save_cat_img').css('color','red');
				$(selectedRow).css('background','');
				$('#wbte_select_single_categories_div').dialog('close');
			});
			
			// Close the dialog without making changes
			$('#wbte_cancelCategory').click(function() {
				$('#wbte_select_single_categories_div').data('selectedRow').css('background','');
				$('#wbte_select_single_categories_div').dialog('close');
			});
		}

		//Dynamic get names
		var cat_filter_name = '#wbte-category-filter-' + page_view;

		//Multiselect Categories
		$(cat_filter_name).attr('multiple','multiple').multiselect({
			noneSelectedText: 'Select Categories',
			selectedList: 4,
			menuHeight: 400,
		}).multiselectfilter(
			{
				width:160,
			}
		);

		var sel_home_cats = $('#cas-form-category-filter-'+page_view + " input[name='categories']").val();
		if ( undefined !== sel_home_cats && sel_home_cats.length > 0 ) {
			sel_home_cats = sel_home_cats.split(',');
			
			$(cat_filter_name + ' option').each(function() {
				if ( '' === $(this).val() ) {
					$(this).prop('selected', false);
				}
				if ($.inArray($(this).val(), sel_home_cats) !== -1) {
					$(this).prop('selected', true);
				}
			});
			
			//Refesh multiselect
			$(cat_filter_name).multiselect("refresh");
			
		}

		//var sales_filter = window.localStorage.getItem('sales_filter');
		var sales_filter = cas_getItemWithExpiration('sales_filter');

		$('#sales_filter').multiselect({
			noneSelectedText: 'Sale status (All)',
			header: false,
			buttonWidth: 160,
			selectedList: 1,
		});

		if ( sales_filter ) {
			$('#sales_filter option').each(function(){
				if ( '' === $(this).val() ) {
					$(this).prop('selected', false);
				}
				if ($(this).val() === sales_filter) {
					$(this).prop('selected', true);
				}
			});
			//Refesh multiselect
			$('#sales_filter').multiselect("refresh");
			$('#sales_filter').change();
		}


		//Multiselect Tags
		$('#product_tag').attr('multiple','multiple').multiselect({
			noneSelectedText: 'Tags (All)',
			selectedList: 4,
			buttonWidth: 200,
		});

	
		$("#product_tag option").each(function() {
			if ( '' === $(this).val() ) {
				$(this).prop('selected', false);
			}
		});

		//var tags_filter = window.localStorage.getItem('cas_tags_filter');
		var tags_filter = cas_getItemWithExpiration('cas_tags_filter');
		if ( tags_filter && tags_filter.length > 0 ) {
			tags_filter = tags_filter.split(',');
			$("#product_tag option").each(function() {
				var value = this.value;
				if ( '' === value ) {
					$(this).prop('selected', false);
				}
				if ($.inArray(value, tags_filter) !== -1) {
					$(this).prop('selected', true);
				}
			});
			$('#product_tag').change();
		} 
		
		//Multiselect Stock status
		$('#stock_status').attr('multiple','multiple').multiselect({
			noneSelectedText: 'Stock Status (All)',
			selectedList: 4,
			buttonWidth: 200,
		});

		$("#stock_status option").each(function() {
			if ( '' === $(this).val() ) {
				$(this).prop('selected', false);
			}
		});

		//var stock_filter = window.localStorage.getItem('cas_stock_filter');
		var stock_filter = cas_getItemWithExpiration('cas_stock_filter');
		if ( stock_filter && stock_filter.length > 0 ) {
			stock_filter = stock_filter.split(',');
			$("#stock_status option").each(function() {
				var value = this.value;
				if ( '' === value ) {
					$(this).prop('selected', false);
				}
				if ($.inArray(value, stock_filter) !== -1) {
					$(this).prop('selected', true);
				}
			});
			$('#stock_status').change();
		} 

		//Multiselect Brands
		$('#product_brand').attr('multiple','multiple').multiselect({
			noneSelectedText: 'Brands (All)',
			selectedList: 4,
			buttonWidth: 200,
		});

	
		$("#product_brand option").each(function() {
			if ( '' === $(this).val() ) {
				$(this).prop('selected', false);
			}
		});

		//var brands_filter = window.localStorage.getItem('cas_brands_filter');
		var brands_filter = cas_getItemWithExpiration('cas_brands_filter');
		if ( brands_filter && brands_filter.length > 0 ) {
			brands_filter = brands_filter.split(',');
			$("#product_brand option").each(function() {
				var value = this.value;
				if ( '' === value ) {
					$(this).prop('selected', false);
				}
				if ($.inArray(value, brands_filter) !== -1) {
					$(this).prop('selected', true);
				}
			});
			$('#product_brand').change();
		} 
			
		//Refesh multiselect
		$("#stock_status").multiselect("refresh");
		$("#product_tag").multiselect("refresh");
		$("#product_brand").multiselect("refresh");
		

		//Row search
		$( '#product_search' ).on( 'keypress', function(event) {
			if (event.key === 'Enter'){
				//window.localStorage.setItem('row_search', $(this).val());
				cas_setItemWithExpiration('row_search', $(this).val());
				casSearchSubmit($('#gsSearch'));
			} 
		} );

		$( '#product_search' ).on( 'input', function() {
			if ( this.value === '' ){
				$( cur_tbl + ' tbody tr' ).show();
				wbtePrintCount(cur_tbl);
				//window.localStorage.setItem('row_search', $(this).val());
				cas_setItemWithExpiration('row_search', $(this).val());
				wbteCalculateFooterTotals();
				wbte_reset_filters();
			}
		});

		//If search exist - apply to new page
		if ( $('#product_search').val() && $('#product_search').val().length > 0 ) {
			rowSearch();
		}

		//Sales filter
		var params = window.location.search;
		if ( 'yes' !== tbl_sale_filter && params.search('sales_filter=') > 0 ) {
			wbteFilterSales();
		}
		if ( params.search('tags-filter=') > 0 || params.search('stock_status=') > 0 ) {
			wbteFilterSales();
		}

		var observer = lozad(); // lazy loads elements
		observer.observe();
		
		var wp_date_format = 'yy-mm-dd';

		$('#datep_from').datepicker( { 'dateFormat': wp_date_format } );
		$('#datep_to').datepicker( { 'dateFormat': wp_date_format } );
		$('input[name="salefrom"]').datepicker(  { 'dateFormat': wp_date_format } );
		$('input[name="saleto"]').datepicker( { 'dateFormat': wp_date_format } );
		
		$('#saveall').css( 'color', 'black' );

		//wbteProgressBar('stop');
		//progressBar('stop');
	}
	wbteProgressBar('stop');
	
});

//Count products
wbtePrintCount(cur_tbl);

async function wbteCountProducts( wbte_table ) {

	var prod     = 0;
	var prod_var = 0;
	prod         = $( wbte_table + ' tbody tr[data-type="prod"]:visible').length;
	prod_var     = $( wbte_table + ' tbody tr[data-type="var"]:visible').length;

	return [ prod, prod_var ];

}
window.wbteCountProducts = function( table ) {
	wbteCountProducts(table);
}

async function wbtePrintCount(cur_tbl) {

	const result = await wbteCountProducts(cur_tbl);
	$('#wbte-prod-count').text( wbte_object.products + ': ' + result[0] + ' | ' + wbte_object.variations + ': ' + result[1] );

}
window.wbtePrintCount = function(tbl) {
	wbtePrintCount(tbl);
}


//Show progressbar when loading
function wbteProgressBar( option ) {

	if (option === 'start') {
		$('#wbte-loading-page').removeClass('wbte-saving').addClass('wbte-saving-show');
		var v = 60;

		setInterval(
			function() {
				if ( v > 100) {
					v = 0;
				}
				$('#pbar-loading-page').val(v);
				v += 20;
			},
			10
		);
	} else {
		$('#wbte-loading-page').removeClass('wbte-saving-show').addClass('wbte-saving');
	}

}
window.wbteProgressBar = function(option) {
	wbteProgressBar(option);
}

/* END ON_LOAD */

/* Local storage functions */

// Set item in local storage with expiration
function cas_setItemWithExpiration(key, value) {
    const now = new Date();
	const ttl = 5 * 60 * 1000; // 5.min

    const item = {
        value: value,
        expiry: now.getTime() + ttl,
    };
    localStorage.setItem(key, JSON.stringify(item));
}
window.cas_setItemWithExpiration = function(key,value) {
	cas_setItemWithExpiration(key,value);
}

// Get item from local storage with expiration
function cas_getItemWithExpiration(key) {
    const itemStr = localStorage.getItem(key);
	const now     = new Date();

    // If the item doesn't exist, return null
    if (!itemStr) {
        return null;
    }

	let item = null;
	try {
		item = JSON.parse(itemStr);
	} catch (e) {
		return null;
	}

    // Compare the expiry time of the item with the current time
    if (now.getTime() > item.expiry) {
        // If the item is expired, delete the item from storage and return null
        localStorage.removeItem(key);
        return null;
    }

    return item.value;
}
window.cas_getItemWithExpiration = function(key){
	cas_getItemWithExpiration(key);
}


function checkAll() {

	var radio = $( "input[name='stype']:checked" ).val();
	var view  = $( '#form-gs-view' ).val();
	var table = tbl;

	switch( view ) {
		case 'ext':
			table = tbl_ext;
			break;
		case 'custom':
			table = tbl_custom;
			break;
		case 'cat':
			table = tbl_cat;
			break;
	}

	//if checked items before bulk calculate (remove other calculations)
	var chkd = $( table + ' tbody tr input[name="id"]:checked' );
	if ( chkd.length > 0 ) {
		$( table + ' tbody tr input[name="id"]:not(:checked)' ).each( function() {
			if ( jQuery(this).is(':visible') ) {
				$(this).prop('checked', false);
				
				if (view === 'prod'){
					setDefaultRowValues(this.form);
				}
			}
		});
		$( '#checkall' ).prop( 'checked', false );
		return 0;
	}

	//bulk calculate
	$( table + ' tbody tr' ).each(
		function(){
			var check = this.children[0].children[0]['id'];
			if ( radio === 'rows' ) {
				if ( jQuery(this).is(':visible') ) {
					$( check ).prop( 'checked', true );
				} else {
					$( check ).prop( 'checked', false );
					if (view === 'prod' ) {
						setDefaultRowValues( this.children[0].children[0] );
					}
				}
			} else {
				$( check ).prop( 'checked', $( '#checkall' ).prop( 'checked' ) );
			}
		}
	);

}
window.checkAll = checkAll;

function checkVisible() {
	var view  = $( '#form-gs-view' ).val();
	var table = tbl;
	
	/*
	if ( view === 'ext' ) {
		table = tbl_ext;
	}*/

	switch( view ) {
		case 'ext':
			table = tbl_ext;
			break;
		case 'custom':
			table = tbl_custom;
			break;
		case 'cat':
			table = tbl_cat;
			break;
	}

	$( table + ' tbody tr' ).each(
		function() {
			var check = this.children[0].children[0]['id'];
			if ( $(this).is(':visible') ) {
				$( check ).prop( 'checked', $( '#checkall' ).prop('checked') );
			} else {
				$( check ).prop( 'checked', false );
			}
		}
	);
}
window.checkVisible = checkVisible;

function cas_get_categories_home( obj, select_name, categories ) {
	event.preventDefault();
	var form               = $('#' + obj.id )[0];
	var filter_name        = select_name.split('-');
	var selected           = $('#wbte-category-filter-'+filter_name[1]).val();
	form[categories].value = selected.filter(n => n).toString();
	form[select_name].remove();
	form.submit();
}
window.cas_get_categories_home = function(obj, select_name, categories) {
	cas_get_categories_home(obj, select_name, categories);
}

function setDefaultRowValues( obj ) {

	obj['stock'].value         = obj['stock'].defaultValue;
	obj['price'].value         = obj['price'].defaultValue;
	obj['saleprice'].value     = obj['saleprice'].defaultValue;
	obj['salefrom'].value      = obj['salefrom'].defaultValue;
	obj['saleto'].value        = obj['saleto'].defaultValue;
	obj['salefrom_time'].value = obj['salefrom_time'].defaultValue;
	obj['saleto_time'].value   = obj['saleto_time'].defaultValue;
	obj['salepercent'].value   = obj['salepercent'].defaultValue;
}
window.setDefaultRowValues = function(obj){
	setDefaultRowValues(obj);
}

function sort( column, type ) {

	var view  = $( '#form-gs-view' ).val();
	var table = tbl;

	switch( view ) {
		case 'ext':
			table = tbl_ext;
			break;
		case 'custom':
			table = tbl_custom;
			break;
		case 'cat':
			table = tbl_cat;
			break;
	}


	//$( table ).tableCalc( 'sortColumn', column, type );
	casDoSort(column,type,table);
	
	if ( $( '#s' + column ).hasClass( 'fa-sort' ) ) {
		$( '#s' + column ).removeClass( 'fa-sort' ).addClass( 'fa-sort-up' );
		resetSortClass( column );
		return;
	}
	if ( $( '#s' + column ).hasClass( 'fa-sort-up' ) ) {
		$( '#s' + column ).removeClass( 'fa-sort-up' ).addClass( 'fa-sort-down' );
		resetSortClass( column );
		return;
	}
	if ( $( '#s' + column ).hasClass( 'fa-sort-down' ) ) {
		$( '#s' + column ).removeClass( 'fa-sort-down' ).addClass( 'fa-sort-up' );
		resetSortClass( column );
		return;
	}

	function resetSortClass( column ){
		for ( var i = 1; i < 10; i++ ) {
			if ( i != column ) {
				$( '#s' + i ).removeClass( 'fa-sort-down' ).removeClass( 'fa-sort-up' ).addClass( 'fa-sort' );
			}
		}
	}

}
window.sort = function(column,type){
	sort(column,type);
}

function casDoSort( column, type, table ){

	var switching = true;
	var shouldSwitch;
	var switchcount = 0;
	var i;
	var dir = 'asc';
	var rows;
	var table = $(table +' > tbody')[0];

	while ( switching ) {

		switching = false;
		rows = table.rows;

		for ( i = 0; i < (rows.length - 1); i++ ) {
		
			shouldSwitch = false;
			
			var x = rows[i].cells[column];
			var y = rows[i + 1].cells[column];

			var node_type = ( x.children.length > 0 && undefined !== x.children[0].nodeName ) ? x.children[0].nodeName : '';
			switch(type) {
				case 'number':
					if( node_type.toLowerCase() == 'input' ){
						x = (x.childNodes[1].value == '' ) ? 0 : parseFloat(x.childNodes[1].value.replace(/[^0-9.-]/g, ''));
						y = (y.childNodes[1].value == '' ) ? 0 : parseFloat(y.childNodes[1].value.replace(/[^0-9.-]/g, ''));
					} else {
						x = ( undefined !== x.innerText ) ? x.innerText.replace( /[^0-9.-]/g, '' ) : '';
						y = ( undefined !== y.innerText ) ? y.innerText.replace( /[^0-9.-]/g, '' ) : '';
					}
					break;
				case 'text':
					if( node_type.toLowerCase() == 'input' ){
						x = x.childNodes[1].value.replace(/^\s+|\s+$/g, '');
						y = y.childNodes[1].value.replace(/^\s+|\s+$/g, '');
					} else {
						x = ( undefined !== x.innerText ) ? x.innerText.replace( /^\s+|\s+$/g, '' ) : '';
						y = ( undefined !== y.innerText ) ? y.innerText.replace( /^\s+|\s+$/g, '' ) : '';
					}
					break;
				case 'date':
					if( node_type.toLowerCase() == 'input' ){
						x = (x.childNodes[1].value == '' ) ? new Date(-25920000000) : new Date(x.childNodes[1].value);
						y = (y.childNodes[1].value == '' ) ? new Date(-25920000000) : new Date(y.childNodes[1].value);
					} else {
						x = (x.innerText == '' ) ? new Date( -25920000000 ) : new Date( x.innerText );
						y = (y.innerText == '' ) ? new Date( -25920000000 ) : new Date( y.innerText );
					}
					break;
				case 'percent':
					if( node_type.toLowerCase() == 'input' ){
						x = (x.childNodes[0].value == '' ) ? 0 : parseFloat(x.childNodes[0].value.replace(/%\s/g, ''));
						y = (y.childNodes[0].value == '' ) ? 0 : parseFloat(y.childNodes[0].value.replace(/%\s/g, ''));
					} else {
						x = x.innerText.replace(/%\s/g, '');
                        y = y.innerText.replace(/%\s/g, '');
					}
					break;
			}

			if ( x === y ) {
				continue;
			}

			if ( dir === "asc" ) {
				if ( x > y ) {
					shouldSwitch = true;
					break;
				}
			} else if ( dir === "desc" ) {
				if ( x < y ) {
					shouldSwitch = true;
					break;
				}
			}
		}

		if ( shouldSwitch ) {
			rows[i].parentNode.insertBefore( rows[i + 1], rows[i] );
			switching = true;
			switchcount++;
		} else {
			if (switchcount == 0 && dir == "asc") {
				dir       = "desc";
				switching = true;
			}
		}

	}

}
window.casDoSort = function(column, type, table){
	casDoSort( column, type, table );
}

function clearSales() {
	
	var is_checked = false;
	
	$( tbl + ' tbody form' ).each(
		function() {
			if (jQuery(this).is(':visible') && this['saleprice'].value > 0 ) {
				this['saleprice'].value     = '';
				this['salefrom'].value      = '';
				this['saleto'].value        = '';
				this['salepercent'].value   = '';
				this['salefrom_time'].value = '00:00';
				this['saleto_time'].value   = '23:59';
				this['id'].checked          = true;
				is_checked = true;
			}
		}
	)

	if ( is_checked ) {
		$( '#saveall' ).css( 'color', 'red' );
		$( '#saveall-foot' ).css( 'color', 'red' );
	} else {
		$( '#saveall' ).css( 'color', '' );
		$( '#saveall-foot' ).css( 'color', '' );
	}
}
window.clearSales = clearSales;


function deleteRows() {
	
	var confirm_delete = confirm( $( '#msg-confirm-delete' ).val() );
	
	if ( confirm_delete === true ) {
		var p_count   = 0;
		var rescount  = 0;
		var count     = 0;
		var nonce_val = $( '#_wpnonce' ).val();
		var json_data = { action: 'wbte_delete_table_rows_data', nonce: nonce_val, rows: [] };

		$( tbl + ' tbody form' ).each(
			function() {
				if ( $( this['id'] ).prop( 'checked' ) ) {
					var frm_data = { id: this['id'].value };
					json_data.rows.push( frm_data );
					count++;
					if ( p_count === 1 ) {
						progressBar('start');
					}
					p_count++;
				}
			}
		)

		$( '#a-clear-sales' ).css( 'color', '' );
		$( '#a-delete-rows' ).css( 'color', 'red' );

		if ( count > 0 ) {
			$.ajax(
				{
					type: 'POST',
					url: ajaxurl,
					datatype: 'json',
					data: json_data,
					success: function( response ) {
						console.log( response );
						rescount++;
						if (rescount > 0) {
							var ms = ( p_count >= 100 ) ? 2000 : 500;
							reloadPage( ms );
						}
					},
					error: function( response ){
						console.log( response );
					}
				}
			);
		
			event.preventDefault();
		}
	} 

}
window.deleteRows = deleteRows;

function saveAll() {

	var count         = 0;
	var p_count       = 0;
	var rescount      = 0;
	var nonce_val     = $( '#_wpnonce' ).val();
	var json_data     = { action: 'wbte_update_table_rows_data', nonce: nonce_val, rows: [] };

	//Check if description is changed
	if ( wbte_object.disable_description !== 'yes' ) {
		$( tbl + ' tbody form').each(
			function() {
				var frm    = this;
				var pId    = frm['id'].value;
				var editor = tinymce.get('desc_'+pId);
				if( editor !== null && editor.isDirty() ) {
					frm['id'].checked = true;
				}
			}
		);
	}
	
	$( tbl + ' tbody form input[name="id"]:checked').each(
		function() {
			var frm  = this.form;
			var pId  = frm['id'].value;
			var desc = ( wbte_object.disable_description !== 'yes' ) ? wp.editor.getContent('desc_'+pId) : '';
			
			var sale_from = frm['salefrom'].value;
			var sale_to   = frm['saleto'].value;
			var utc_from  = frm['salefrom'].dataset.utcDate;
			var utc_to    = frm['saleto'].dataset.utcDate;

			var utcDatePattern = /^\d{4}-\d{2}-\d{2}$/;

			// Validate sale_from date
			if ('' !== sale_from && !utcDatePattern.test(sale_from)) {
				sale_from = utc_from;
			}
		 
			// Validate sale_to date
			if ('' !== sale_to && !utcDatePattern.test(sale_to)) {
				sale_to = utc_to;
			}
			
			var frm_data = { id: frm['id'].value,
				stock: frm['stock'].value, price: frm['price'].value,
				saleprice: frm['saleprice'].value, salefrom: sale_from, saleto: sale_to,
				salefrom_time: frm['salefrom_time'].value, saleto_time: frm['saleto_time'].value,
				name: frm['product_name'].value, name_update: frm['name_update'].value, description: desc
			};

			if ( custom_col_active && 'yes' === custom_col_active && show_sku_home === 'no' ) {
				frm_data['customprice'] = frm['customprice'].value;
			} else if ( show_sku_home === 'yes' ) {
				frm_data['sku'] = frm['sku'].value;
			}

			//Vendor
			if ( 'yes' === vendor_active ) {
				frm_data['vendor'] = frm['vendor_selected'].value;
			}

			//Brands
			if ( 'yes' === wbte_object.brand_active ) {
				frm_data['brands'] = frm['brand_input'].value;
			}
			
			json_data.rows.push( frm_data );
			count++;
			if ( count === 1 ) {
				$( '#saveall' ).attr( 'disabled', true );
				$( '#saveall_foot' ).attr( 'disabled', true );
			}

			if( count === 80 ) {
				sendJson( ajaxurl, json_data );
				json_data.rows = [];
				count = 0;
				setTimeout(function(){console.log(p_count);},800);
			}
			
			if ( p_count === 1 ) {
				progressBar('start');
			}
			p_count++;
		
		}
	);


	$.ajax(
		{
			type: 'POST',
			url: ajaxurl,
			datatype: 'json',
			data: json_data,
			success: function( response ) {
				console.log( response );
				rescount++;
				if (rescount > 0) {
					jQuery( '#saveall' ).attr( 'disabled', false );
					jQuery( '#saveall_foot' ).attr( 'disabled', false );
					var ms = ( p_count >= 100 ) ? 3000 : 500;
					reloadPage( ms );
				}
			},
			error: function( response ){
				progressBar('stop');
				console.log( response );
			}
		}
	);
	

	event.preventDefault();

};
window.saveAll = saveAll;

function saveAllExt() {

	var count         = 0;
	var p_count       = 0;
	var rescount      = 0;
	var nonce_val     = $( '#_wpnonce' ).val();
	var json_data     = { action: 'wbte_update_table_rows_ext_data', nonce: nonce_val, rows: [] };

	//Check if description is changed
	if ( wbte_object.disable_description !== 'yes' ) {
		$( tbl_ext + ' tbody form').each(
			function() {
				var frm    = this;
				var pId    = frm['id'].value;
				var editor = tinymce.get('desc_'+pId);
				if( editor !== null && editor.isDirty() ) {
					frm['id'].checked = true;
				}
			}
		);
	}
	
	$( tbl_ext + ' tbody form input[name="id"]:checked' ).each(
		function() {
		
			var frm  = this.form;
			var pId  = frm['id'].value;
			var desc = ( wbte_object.disable_description !== 'yes' ) ? wp.editor.getContent('desc_'+pId) : '';
			
			var frm_data = { id: frm['id'].value,
				featured: frm['featured'].value, sku: frm['sku'].value,
				backorder: frm['backorder'].value, instock: frm['instock'].value, weight: frm['weight'].value,
				length: frm['_length'].value, width: frm['width'].value, height: frm['height'].value,
				tags: frm['tags'].value, visibility: frm['visibility'].value,
				name: frm['product_name'].value, name_update: frm['name_update'].value,
				product_img: frm['product_img'].value, parent_id: frm['parent_id'].value,
				description: desc, shipping: frm['shipping'].value
			};
			
			json_data.rows.push( frm_data );
			count++;
			if ( count === 1 ) {
				jQuery( '#saveallExt' ).attr( 'disabled', true );
				jQuery( '#saveall_foot_ext' ).attr( 'disabled', true );
			}

			if( count === 50 ) {
				sendJson( ajaxurl, json_data );
				json_data.rows = [];
				count = 0;
				setTimeout(function(){console.log(p_count);},1000);
			}

			if ( p_count === 1 ) {
				progressBar('start');
				jQuery('.wbte-bulk-images').css('display','none');
			}
			p_count++;

		}
	);

	$.ajax(
		{
			type: 'POST',
			url: ajaxurl,
			datatype: 'json',
			data: json_data,
			success: function( response ) {
				//console.log( response );
				rescount++;
				if (rescount > 0) {
					jQuery( '#saveallExt' ).attr( 'disabled', false );
					jQuery( '#saveall_foot_ext' ).attr( 'disabled', false );
					var ms = ( p_count >= 100 ) ? 3000 : 800;
					reloadPage( ms );
				}
			},
			error: function( response ){
				progressBar('stop');
				console.log( response );
				jQuery( '#saveallExt' ).attr( 'disabled', false );
				jQuery( '#saveall_foot_ext' ).attr( 'disabled', false );
				jQuery('.wbte-bulk-images').css('display','block');
			}
		}
	);
	

	event.preventDefault();

};
window.saveAllExt = saveAllExt;

function saveCatImgAll() {

	var nonce_val = $( '#_wpnonce' ).val();
	var json_data = { action: 'wbte_update_category_image_data', nonce: nonce_val, rows: [] };

	$( tbl_cat + ' tbody form input[name="id"]:checked' ).each( function() {
		
		var frm  = this.form;
		
		var frm_data = { id: frm['id'].value,
			name: frm['product_name'].value, 
			name_update: frm['name_update'].value,
			product_img: frm['product_img'].value, 
			product_img_ids: frm['product_img_ids'].value,
			parent_id: frm['parent_id'].value,
			categories: frm['product_cat_ids'].value,
		};
		
		json_data.rows.push( frm_data );
		
		if ( json_data.rows.length === 1 ){
			progressBar('start');
		}
	});

	if ( json_data.rows.length > 0 ) {

		$.ajax(
			{
				type: 'POST',
				url: ajaxurl,
				datatype: 'json',
				data: json_data,
				success: function( response ) {
					//console.log( response );
					progressBar('stop');
					reloadPage(300);
				},
				error: function( response ){
					progressBar('stop');
					console.log( response );
				}
			}
		);
	}
	

	event.preventDefault();

}
window.saveCatImgAll = saveCatImgAll;

function sendJson( ajaxurl, json_data ) {

	$.ajax(
		{
			type: 'POST',
			url: ajaxurl,
			datatype: 'json',
			data: json_data,
			success: function( response ) {
				console.log( response );
			},
			error: function( response ){
				console.log( response );
			}
		}
	);

}
window.sendJson = function( ajaxurl, json_data){
	sendJson(ajaxurl,json_data);
}

function reloadPage( msec ) {

	setTimeout(
		function() {
			window.location.reload();
		},
		msec
	);

}
window.reloadPage = function(msec){
	reloadPage(msec);
}

function setSalesDate( obj ) {

	if ( ! obj ) {
		return;
	}

	$( tbl + ' tbody form' ).each(
		
		function() {
			if ( obj.id === 'datep_from' ) {
				this[ 'salefrom' ].value = obj.value;
			}
			if ( obj.id === 'datep_to' ) {
				this[ 'saleto' ].value = obj.value;
			}
		}
	);

	$( '#saveall' ).css( 'color', aColor );
	$( '#checkall' ).prop( 'checked', true );

	checkAll();

}
window.setSalesDate = function(obj){
	setSalesDate(obj);
}

function setSalesTime( obj ) {
	
	if ( ! obj ) {
		return;
	}

	$( tbl + ' tbody form' ).each(
		
		function() {
			if ( obj.id === 'datep_from_time' ) {
				this[ 'salefrom_time' ].value = obj.value;
			}
			if ( obj.id === 'datep_to_time' ) {
				this[ 'saleto_time' ].value = obj.value;
			}
		}
	);

	$( '#saveall' ).css( 'color', aColor );
	$( '#checkall' ).prop( 'checked', true );

	checkAll();
}
window.setSalesTime = function(obj){
	setSalesTime(obj);
}

function calcChanged( ft ) {

	var tot         = sum_col;
	var is_checkbox = ( this.name === 'id' ) ? true : false;
	var is_inputbox = ( this.type === 'number' || this.type === 'text' || this.type === 'time' || this.type === 'select-one' || this.type === 'textarea' ) ? true : false;
	var ftv         = ( ft === true ) ? true : false;

	if ( ! ftv ) {
		if ( is_checkbox ) {
			$( this ).prop( 'checked', this.checked );
		}

		if ( is_inputbox ) {
			this.form['id'].checked = true;
		}

	} else {
		findSalePricePercent();
	}

	$( '#saveall' ).css( 'color', aColor );

	//Stock and stock value
	wbteCalculateFooterTotals();

}
window.calcChanged = function(ft){
	calcChanged(ft);
}

function wbteCalculateFooterTotals() {
	
	if ( $( tbl + ' tbody tr:visible' ).length > 0 ) {
		var stock_sum = 0;
		var total_sum = 0;
		$( tbl + ' tbody tr:visible' ).each( function() {
			var frm   = this.children[0].children[0];
			var items = frm.elements;
			stock_sum += ( items['stock'].value !== '' ) ? parseInt( items['stock'].value ) : 0;
			total_sum += ( items['totcol'].value !== '' ) ? parseFloat( items['totcol'].value ) : 0;
		});
		$( '#tbl-total-f' ).text( parseFloat( total_sum ).toFixed(2) );
		$( '#tbl-total-s' ).text( stock_sum );
		
	}
}
window.wbteCalculateFooterTotals = wbteCalculateFooterTotals;

function extChanged() {

	if ( undefined === this.form ) return;

	if ( this.name === 'featured' ) {
		if ( $(this).prop('checked') === true ) {
			$( this ).val( 1 );
		} else {
			$( this ).val( '' );
		}
	}

	if ( this.name !== 'id' ) {
		this.form['id'].checked = true;
	}

}
window.extChanged = extChanged;

function changeCalcType( calcType, customCalculation ){

	$( tbl ).tableCalc( 'calculate', calcType, customCalculation );
	calcChanged();

};
window.changeCalcType = function(calcType, customCalculation){
	changeCalcType(calcType, customCalculation);
}

function calcSpecial( i, o, id ) {

	var sign    = '+';
	var in_val  = Number( $( '#' + id ).val() );
	var in_type = $( '#' + id + '_type option:selected' ).val();

	if ( in_val === 0 && 
		in_type !== 'round_up' && 
		in_type !== 'round_down' && 
		in_type !== 'round_two' && 
		in_type !== 'fix_n' ) {

		if ( in_type === 'prev_price' || in_type === 'clear_price' || in_type === 'prev_sale_price' || in_type === 'clear_stock'  ) {
			wbteFixPrice( in_type );
		}
		$( '#' + id + '_type' ).val( '0' );

		return;
	}

	var calctype = '';

	switch ( in_type ) {
		case '0':
			break;
		case 'up_p':
			sign     = '+';
			calctype = 'percent';
			break;
		case 'up_s_p': //Saleprice
			sign     = '+';
			calctype = 'percent';
			i        = 2; //tablecalc number
			break;
		case 'up_n':
			sign     = '+';
			calctype = 'number';
			break;
		case 'up_s_n': //Saleprice
			sign     = '+';
			calctype = 'number';
			i        = 2;
			break;
		case 'down_p':
			sign     = '-';
			calctype = 'percent';
			break;
		case 'down_s_p': //Saleprice
			sign     = '-';
			calctype = 'percent';
			i        = 2;
			break;
		case 'down_n':
			sign     = '-';
			calctype = 'number';
			break;
		case 'down_s_n': //Saleprice
			sign     = '-';
			calctype = 'number';
			i        = 2;
			break;
		case 'fix_n':
			sign     = '=';
			calctype = 'number';
			break;
		case 'round_up':
			sign     = 'u';
			calctype = 'round';
			break;
		case 'round_down':
			sign     = 'd';
			calctype = 'round';
			break;
		case 'round_two':
			sign     = 't';
			calctype = 'round';
			break;

	}

	$( '#' + id ).val( '' );
	$( '#' + id + '_type' ).val( '0' );

	var percent = 1.0;
	if ( sign === '+' ) {
		percent = percent + in_val / 100;
	} else if ( sign === '-' ) {
		percent = percent - in_val / 100;
	}
	var formula = '(0:' + i + '*' + percent + ')==0:' + o + '';

	if ( calctype === 'number' ) {
		if ( sign === '=' ) {
			var c = o - 2;
			switch(o){
				case 7:
					c = 3;
					break;
				case 8:
					c = 4;
					break;
			}
			formula = '(0:' + c + '*' + 0 + '+' + in_val + ')==0:' + o + '';
		} else {
			formula = '(0:' + i + sign + in_val + ')==0:' + o + '';
		}
	}

	if ( calctype === 'round' ) {
		roundNumbers( o, sign );
	} else {
		$( tbl ).tableCalc( 'calculate', 'c', formula );
	}

	$( tbl ).tableCalc( 'calculate', 'c', '(0:0 * 0:1)' );

	if ( 'sale_price_select' === id ) {
		findSalePricePercent();
	}

	if ( $( tbl + ' tbody tr:visible' ).length > 0 ) {
		$( '#tbl-total-f' ).text( $( tbl ).tableCalc( 'getSum', sum_col ) );
	}

	$( '#saveall' ).css( 'color', aColor );
	$( '#checkall' ).prop( 'checked', true );

	checkAll();

};
window.calcSpecial = function(i, o, id){
	calcSpecial(i, o, id);
}

function wbteFixPrice( element ) {
	
	//if checked items before bulk calculate - do only caclulations for those
	var chkd     = $( '#wbtetable tbody tr input[name="id"]:checked' );
	var onlyChkd = (chkd.length > 0) ? ' input[name="id"]:checked' : '';

	//editor home
	$( '#wbtetable tbody tr' + onlyChkd ).each(
		function() {
			var frm          = (chkd.length > 0) ? this.form : this.children[0].children[0];
			var frm_elements = frm.elements;

			if ( $(this).is(':visible') ) {

				if ( 'prev_price' === element ) {

					if ( frm_elements['prev_price'].value > 0 ) {
						//Copy prev price
						frm_elements['price'].value = frm_elements['prev_price'].value;
						
						//Changed
						$( frm_elements['id'] ).prop( 'checked', true );
					}

				} else if ( 'clear_price' === element ) {
					
					//Clear price
					frm_elements['price'].value = '';
					
					//Changed
					$( frm_elements['id'] ).prop( 'checked', true );
					
				} else if ( 'prev_sale_price' === element ) {
					
					if ( frm_elements['prev_sale_price'].value > 0 ) {

						//Copy prev sale price
						frm_elements['saleprice'].value = frm_elements['prev_sale_price'].value;
						
						//Changed
						$( frm_elements['id'] ).prop( 'checked', true );

					}
				} else if ( 'clear_stock' === element ) {
					//Clear stock
					frm_elements['stock'].value = '';
					
					//Changed
					$( frm_elements['id'] ).prop( 'checked', true );
				}


			}
		}
	);

	if ( 'prev_sale_price' === element ) {
		//Calculate sale percent
		findSalePricePercent();
	}
}
window.wbteFixPrice = function(e){
	wbteFixPrice(e);
}

function bulkSetValues( id, the_table ) {
	
	var view      = $( '#form-gs-view' ).val();
	var in_val    = $( '#' + id ).val();
	var in_type   = $( '#' + id + '_type option:selected' ).val();
	var table     = tbl;
	var delimiter = wbte_object.sku_delimiter;

	$( '#' + id ).val( '' );
	$( '#' + id + '_type' ).val( '0' );

	switch( view ) {
		case 'ext':
			table = tbl_ext;
			break;
		case 'custom':
			table = tbl_custom;
			break;
		case 'cat':
			table = tbl_cat;
			break;
	}

	var is_changed = false;

	var brands_arr = [];
	if ( 'yes' === wbte_object.brand_filter ) {
		$('#product_brand_select option:selected').each(function(){
			if ( '' !== this.value ) {
				brands_arr.push( this.value );
			}
		});
	}

	var category_arr      = [];
	var category_arr_full = [];
	if ( 'cat' === view ) {
		$('#bulk_product_cat option').each(function(){
			if ( '' !== this.value ) {
				if ( this.selected ) {
					category_arr.push( { id: this.value, name: this.text.trim() } );
				}
				category_arr_full.push( { id: this.value, name: this.text.trim() } );
			}
		});
	}

	//if checked items before bulk calculate - do only caclulations for those
	var chkd     = $( table + ' tbody tr input[name="id"]:checked' );
	var onlyChkd = (chkd.length > 0) ? ' input[name="id"]:checked' : '';

	$( table + ' tbody tr' + onlyChkd ).each(
		function() {
			
			var frm          = (chkd.length > 0) ? this.form : this.children[0].children[0];
			var frm_elements = frm.elements;

			if ( $(this).is(':visible') ) {

				switch ( id ) {
					case 'featured_select':
						if ( in_type === 'yes' ) {
							$( frm_elements['featured'] ).prop( 'checked', true );
							frm_elements['featured'].value = '1';
						} else {
							$( frm_elements['featured'] ).prop( 'checked', false );
							frm_elements['featured'].value = '';
						}
						break;
					case 'sku_select':
						if ( in_type === 'replace' ) {
							frm_elements['sku'].value = in_val;
						} else if ( in_type === 'add-after' ) {
							frm_elements['sku'].value += in_val; 
						} else if ( in_type === 'add-before' ) {
							frm_elements['sku'].value = in_val + frm_elements['sku'].value;
						} else if ( in_type === 'add-id-after' ) {
							frm_elements['sku'].value += delimiter + frm_elements['id'].value; 
						} else if ( in_type === 'add-id-before' ) {
							frm_elements['sku'].value = frm_elements['id'].value + delimiter + frm_elements['sku'].value;	
						} else if ( in_type === 'add-weight' ) {
							frm_elements['sku'].value += delimiter + frm_elements['weight'].value; 
						} else if ( in_type === 'add-length' ) {
							frm_elements['sku'].value += delimiter + frm_elements['_length'].value;
						} else if ( in_type === 'add-width' ) {
							frm_elements['sku'].value += delimiter + frm_elements['width'].value;
						} else if ( in_type === 'add-height' ) {
							frm_elements['sku'].value += delimiter + frm_elements['height'].value;
						} else if ( in_type === 'clear' ) {
							frm_elements['sku'].value = '';
						} else if ( in_type === 'generate' ) {
							frm_elements['sku'].value = generateSku(frm_elements['product_name'].value, delimiter);
						} else if ( in_type === 'toupper' ) {
							frm_elements['sku'].value = frm_elements['sku'].value.toUpperCase();
						} else if ( in_type === 'tolower' ) {
							frm_elements['sku'].value = frm_elements['sku'].value.toLowerCase();
						}
						break;
					case 'tags_select':
						if ( in_type === 'add' ) {
							if ( frm_elements['tags'].value.trim() === '' ) {
								frm_elements['tags'].value += in_val;
							} else {
								frm_elements['tags'].value += ', ' + in_val;
							}
						} else if ( in_type === 'toupper' ) {
							frm_elements['tags'].value = frm_elements['tags'].value.toUpperCase(); 
						} else if ( in_type === 'tolower' ) {
							frm_elements['tags'].value = frm_elements['tags'].value.toLowerCase(); 
						} else if ( in_type === 'clear' ) {
							frm_elements['tags'].value = ''; 
						} else if ( in_type === 'replace' ) {
							frm_elements['tags'].value = in_val; 
						}
						break;
					case 'weight_select':
						var current = frm_elements['weight'].value || '0';
						frm_elements['weight'].value = getMeasure( in_type, in_val, current );
						break;
					case 'length_select':
						var current = frm_elements['_length'].value || '0';
						frm_elements['_length'].value = getMeasure( in_type, in_val, current );
						break;
					case 'width_select':
						var current = frm_elements['width'].value || '0';
						frm_elements['width'].value = getMeasure( in_type, in_val, current );
						break;
					case 'height_select':
						var current = frm_elements['height'].value || '0';
						frm_elements['height'].value = getMeasure( in_type, in_val, current );
						break;
					case 'backorder_select':
						$( frm_elements['backorder'] ).val(in_type).change();
						break;
					case 'instock_select':
						$( frm_elements['instock'] ).val(in_type).change();
						break;
					case 'visibility_select':
						$( frm_elements['visibility'] ).val(in_type).change();
						break;
					case 'vendor_select':
						$( frm_elements['vendor_selected'] ).val(in_type).change();
						break;	
					case 'shipping_select':
						$( frm_elements['shipping'] ).val(in_type).change();
						break;	
					case 'brand_select':
						//Get selected brands
						setBrand( in_type, frm_elements );
						$( frm_elements['brand_input'] ).show();
						$('#cas-brand-div-' + frm_elements['id'].value ).hide();
						break;
					case 'bulkcat_select':
						wbte_setCategory( in_type, frm_elements, category_arr, category_arr_full );
						break;
				}

				function setBrand( type, frm_elements ) {
					var currents = ($( frm_elements['brand_input_hidden'] ).val()) ? $( frm_elements['brand_input_hidden'] ).val().split(',') : [];
					switch(type){
						case 'add':
							$( frm_elements['brand_input'] ).val( currents.concat(brands_arr).toString() );
							break;
						case 'set':
							$( frm_elements['brand_input'] ).val( brands_arr.toString() );
							break;
						case 'remove':
							var brands_arr_set = new Set(brands_arr);
							var result_arr     = currents.filter(brand => !brands_arr_set.has(brand));
							$( frm_elements['brand_input'] ).val( result_arr.toString() );
							break;
						case 'clear':
							$( frm_elements['brand_input'] ).val('')
							break;
					}
				}
				
				function getMeasure( type, value, oldvalue ) {
					var retval = 0;
					switch ( type ) {
						case 'fix_n':
							retval = value;
							break;
						case 'up_n':
							retval = eval( parseFloat( oldvalue ) + parseFloat( value ) );
							break;
						case 'down_n':
							retval = eval( parseFloat( oldvalue ) - parseFloat( value ) );
							break;
						case 'clear':
							retval = '';
					}
					return retval;
				}

				function generateSku( name, delimiter ) {
					
					if ( name === undefined || name === '' ) {
						return '';
					}

					var sku_count   = ( parseInt( wbte_object.sku_count ) > 0 ) ? parseInt( wbte_object.sku_count ) : 3;
					var isVariation = name.search(',');
					var nameArr     = ( isVariation === -1 ) ? name.split( ' ' ) : name.split(',');
					var retval      = '';
					if ( nameArr.length > 0 ) {
						for ( var i = 0; i < nameArr.length; i++ ) {
							retval += nameArr[i].trim().substring( 0, sku_count ) + delimiter;
						}
						retval = retval.substring( 0, parseInt(retval.length - 1) );
					} else {
						retval = name.substring( 0, sku_count );
					}
					return retval;
				}

				//Row changed = true
				$( frm_elements['id'] ).prop( 'checked', true );
				is_changed = true;
			} else {
				//Not changed
				$( frm_elements['id'] ).prop( 'checked', false );
			}
		}
	)
	if (is_changed) {
		$( '#checkall' ).prop( 'checked', true );
		checkAll();
	}
	

}
window.bulkSetValues = function(id, the_table){
	bulkSetValues(id, the_table);
}

function wbte_setCategory( type, frm_elements, category_arr, category_arr_full ) {
					
	var cat_ids = ($(frm_elements['product_cat_ids']).val()) ? $(frm_elements['product_cat_ids']).val().split(',').map(function(id) {
		// Find the category in category_arr_full with the same value as the current id
		var category = category_arr_full.find(function(cat) {
			return cat.id === id;
		});
		return category ? { id: id, name: category.name } : { id: id, name: '' };
	}) : [];
	
	switch(type) {
		case 'add':
			var newCategories = category_arr.filter(function(catToAdd) {
				// Check if catToAdd is not already in cat_ids
				return !cat_ids.some(function(existingCat) {
					return existingCat.id === catToAdd.id;
				});
			});
			cat_ids = cat_ids.concat(newCategories.map(function(cat) {
				return { id: cat.id, name: cat.name };
			}));
			break;
		case 'set':
			cat_ids = category_arr.map(function(cat) {
				return { id: cat.id, name: cat.name };
			});
			break;
		case 'remove':
			// Create a Set of IDs from category_arr for efficient lookup
			var catsIdSet = new Set(category_arr.map(cat => cat.id));
			cat_ids       = cat_ids.filter(cat => !catsIdSet.has(cat.id));
			break;
		case 'clear':
			cat_ids = [];
			break;
	}
	
	// Convert cat_ids to just ids for the input value.
	frm_elements['product_cat_ids'].value = cat_ids.map(function(cat) { return cat.id; }).toString();
	
	// Clear the existing category list.
	$('#category_list_' + frm_elements['id'].value).empty();
	
	// Iterate over cat_ids
	cat_ids.forEach(function(element) {
		$('#category_list_' + frm_elements['id'].value).append(
			'<div id="link_' + element.id + '_' + frm_elements['id'].value + '"><a href="#" onclick="wbteRemoveCat(\'' + element.id + '\','  + frm_elements['id'].value + ');"><i class="fas fa-check-square"></i> ' + element.name + '</a></div>'
		);
	});
	
}
window.wbte_setCategory = function(type,frm_elements,category_arr,category_arr_full){
	wbte_setCategory( type, frm_elements, category_arr, category_arr_full );
}

function wbteRemoveCat(itemId, prodId) {

	$('#link_'+itemId+'_'+prodId).remove();

    var ids = $('#product_cat_ids_' + prodId).val() ? $('#product_cat_ids_' + prodId).val().split(',') : [];
    
    ids = ids.filter(function(id) {
        return id.trim() !== itemId.toString();
    });
    
    $('#product_cat_ids_' + prodId).val(ids.join(','));
	$('#category_list_' + prodId).closest('tr').find('input[name="id"]').prop('checked', true);

}
window.wbteRemoveCat = function(itemId, prodId){
	wbteRemoveCat(itemId, prodId);
}

function wbteDeleteCategories( id ) {
	$('#product_cat_ids_'+id).val('');
	$('#category_list_'+id).empty();
	$(tbl_cat+' tbody tr form input[name="id"][value="'+id+'"]').prop('checked',true);
}
window.wbteDeleteCategories = function(id) {
	wbteDeleteCategories(id);
}

function wbteZoomImagesBulk(){
	$(cur_tbl + ' tbody tr:visible').each(function(){
		var id = this.children[0].id;
		wbteZoomImages(id);
	});
}
window.wbteZoomImagesBulk = wbteZoomImagesBulk;

function wbteZoomImages( id ) {
	if ( $('#images_container_'+id+' img').hasClass('wbte-thumb-cust') ) {
		$('#images_container_'+id+' img').removeClass('wbte-thumb-cust').addClass('wbte-thumb-cust-large');
	} else {
		$('#images_container_'+id+' img').removeClass('wbte-thumb-cust-large').addClass('wbte-thumb-cust');
	}
}
window.wbteZoomImages = function(id) {
	wbteZoomImages(id);
}

function roundNumbers( column, sign ) {

	var table = $( tbl + ' tbody tr' );

	table.each(
		function() {
			var rValue = this.children[column].children[0].value;
			var rOut   = 0;
			switch ( sign ) {
				case 'd':
					rOut = Number(Math.floor(rValue)).toFixed(2);
					break;
				case 'u':
					rOut = Number(Math.ceil(rValue)).toFixed(2);
					break;
				case 't':
					rOut = Number(rValue).toFixed(1);
					break;
			}

			var newValue = Number(rOut).toFixed(2);
			if ( Number(rOut) === 0 ) {
				newValue = '';
			}
			
			this.children[column].children[0].value = newValue;
		}
	)
}
window.roundNumbers = function(column,sign){
	roundNumbers(column,sign);
}

function casRangeTypeChanged(bte_min, bte_max, onChange) {
	
	const table = '#wbtetable';
	var selVal  = $('#cas-range-type option:selected').val();

	//Remeber 
	var getSelValue = cas_getItemWithExpiration('cas-range-type-value') || '';
	if (onChange || '' === getSelValue) {
		cas_setItemWithExpiration('cas-range-type-value',selVal);
	}

	if (! onChange && '' !== getSelValue) {
		selVal = getSelValue;
		$('#cas-range-type').val(selVal);
	}

	var min_max_arr = [];
	$( table +' tbody tr:visible').each(function(){
		var row   = $(this);
		var value = ( isNaN( parseFloat(row.find('td input[name="' + selVal + '"]').val() ) )) ? 0 : parseFloat( row.find('td input[name="' + selVal + '"]').val() );
		min_max_arr.push( value );
	});

	var min_val  = ('' === bte_min) ? Math.min(...min_max_arr) : bte_min;
	var max_val  = ('' === bte_max) ? Math.max(...min_max_arr)  : bte_max;
	var rangeSet = false;

	$( '#cas-slider-range' ).slider({
		range: true,
		min: 0,
		max: Math.max(...min_max_arr) * 1.5,
		values: [ min_val, max_val ],
		slide: function( event, ui ) {
			$( '#cas-amount' ).val( ui.values[ 0 ] + " - " + ui.values[ 1 ] );
			casFilterRange( selVal, table, ui.values[0], ui.values[1] );
			rangeSet = true;
		}
	});

	if ( '' === bte_min && ! rangeSet ) {
		casSliderUpdater(min_val, max_val);
	}
}
window.casRangeTypeChanged = function(bte_min, bte_max, onChange){
	casRangeTypeChanged(bte_min, bte_max, onChange);
}

function casFilterRange(filterColumn, table, minValue, maxValue) {
	
	var visibleRows = $(table +' tbody').find('tr:visible');

	$(table + ' tbody').find('tr').each(function() {
	 
	  var row = $(this);
  
	  // Get the value of the column to filter on
	  var value = parseFloat(row.find('td input[name="' + filterColumn + '"]').val());
  
	  // Hide or show the row based on the filter range
	  if (value >= minValue && value <= maxValue) {
		$(row).show();
	  } else {
		$(row).hide();
	  }

	});

	// If other filters are applied, make sure to keep them applied
	visibleRows.each(function() {
		var row = $(this);
		if (!row.is(':visible')) {
		  // If this row was previously hidden, hide it again
		  row.hide();
		}
	});
}
window.casFilterRange = function(filterColumn, table, minValue, maxValue){
	casFilterRange(filterColumn, table, minValue, maxValue);
}

function casSliderUpdater(min, max) {
	if ( Infinity === min || Infinity === max ) return;
	$( '#cas-slider-range' ).slider('values', [ min, max ] );
	$( '#cas-amount' ).val( min + " - " + max );
}
window.casSliderUpdater = function(min,max){
	casSliderUpdater(min,max);
}

function casSearchSubmit( obj ) {
	
	var radio = $( "input[name='stype']:checked" ).val();
	var input = $( '#product_search' ).val();

	switch( radio ) {
		case 'rows':
			event.preventDefault();
			( '' === input ) ? obj.submit() : rowSearch(); 
			break;
		default:
			obj.submit();
			break;
	}

}
window.casSearchSubmit = function(obj){
	casSearchSubmit(obj);
}

function rowSearch(){

	var view  = $( '#form-gs-view' ).val();
	var input = $( '#product_search' ).val();
	var table = tbl; //$( tbl + ' tbody tr' );
	var radio = $( "input[name='stype']:checked" ).val();
	var arr   = [];
	
	switch( view ) {
		case 'ext':
			table = tbl_ext;
			break;
		case 'custom':
			table = tbl_custom;
			break;
		case 'cat':
			table = tbl_cat;
			break;
	}
	
	//make sure no checked rows
	if ( input.length > 0 ) {
		$( table + ' tbody tr input[name="id"]' ).prop('checked',false);
	}

	if ( radio === 'rows' ) {
		if ( input.length > 0 ) {
			input = input.toLowerCase().replace( ',', '' );
			arr   = input.split( ' ' );
		}

		var p_arr  = [];
		var selVal = $('#cas-range-type option:selected').val();
		
		$( table + ' tbody tr' ).each( function() {
			
				var frm_elements  = this.children[0].children[0].elements;
				var textValue     = frm_elements['product_name'].value;
				var skuPriceValue = '';
				var p_id          = frm_elements['id'].value;
				var desc          = '';
				var sku           = '';

				if ( 'prod' === view || 'ext' === view ) {
					skuPriceValue = ( 'ext' === view ) ? frm_elements['sku'].value : frm_elements['price'].value;
					sku           = ( 'ext' === view ) ? frm_elements['sku'].value : '';

					//Show description
					if ( wbte_object.disable_description !== 'yes' ){
						desc = this.children[1].children[0].children[2].getElementsByTagName('textarea')[0].innerText;
						sku  = this.children[1].children[0].children[3].children[1].innerText;
					}
				}
				
				if (textValue.length > 0) {
					
					var txt   = textValue.toLowerCase();
					sku       = ( 'prod' === view && sku.length > 5 ) ? sku.toLowerCase().split(':')[1] : sku; 
					var count = 0;

					for( var i = 0; i < arr.length; i++ ) {
						if ( txt.match( arr[i] ) || 
							 skuPriceValue.toLowerCase().match( arr[i] ) || 
							 desc.toLowerCase().match( arr[i] ) || 
							 sku.match( arr[i] ) ||
							 p_id.trim().match( arr[i] )
							 ) {
								count++;
							}
					}
					if ( count === arr.length ) {
						if ( 'prod' === view ) {
							p_arr.push( frm_elements[selVal].value );
						}
						$(this).show();
					} else {
						$(this).hide();
					}
				} 
			}
		);
		
		//Count rows
		wbtePrintCount(table);

		//Stock and stock value
		wbteCalculateFooterTotals();

		//Update Range filter
		if ( 'prod' === view ) {
			casSliderUpdater( Math.min(...p_arr), Math.max(...p_arr) );
		}

	} 

}
window.rowSearch = rowSearch;

function sortTable( table, order ) {
	
	var sort  = 'asc';
	var tbody = table.find( 'tbody' );
	if ( order !== 'asc' ) {
		sort = '';
	}

    tbody.find('tr').sort(function(a, b) {
		var a1 = a.children[1].children[0].value;
		var b1 = b.children[1].children[0].value;
	
        if ( sort === 'asc' ) {
			return a1.localeCompare(b1);
        } else {
			return b1.localeCompare(a1);
        }
	}).appendTo(tbody);
	
}
window.sortTable = function(table,order){
	sortTable(table,order);
}

function findSalePricePercent() {
	
	var table = $( tbl + ' tbody form' );

	table.each(
		function() {
			calcPercent( this, true );
		}
	);
}
window.findSalePricePercent = findSalePricePercent;

function calcPercent( obj, isForm = false ) {

	var form;

	if ( isForm ) {
		form = obj;
	} else {
		form = obj.form;
	}

	var price      = form['price'].value;
	var sale_price = form['saleprice'].value;

	if ( parseFloat(price) > 0 && parseFloat(sale_price) > 0 ) {
		var percent = eval( ( price - sale_price ) * 100 / price );
		form['salepercent'].value = String( Number( percent ).toFixed(0) ) + '% ';
		if ( percent < 0 ) {
			form['salepercent'].style.color = 'red';
		} else {
			form['salepercent'].style.color = 'black';
		}
	}

}
window.calcPercent = function(obj, isForm = false){
	calcPercent(obj,isForm);
}

function progressBar( option ) {
	
	if (option === 'start') {
		$('#wbte-saving').removeClass('wbte-saving').addClass('wbte-saving-show');
		var v = 0;
		
		setInterval(
			function() {
				if ( v > 100) {
					v = 0;
				}
				$('#pbar-saving').val(v);
				v += 5;
			},
			50
		);
	} else {
		$('#wbte-saving').removeClass('wbte-saving-show').addClass('wbte-saving');
	}

}
window.progressBar = function(option){
	progressBar(option);
}

var frame;

function openMediaLib( product_id ) {
	
	event.preventDefault();
	
	var id       = product_id;
	var w_title  = wbte_object.media_lib_title;
	var w_button = wbte_object.media_lib_button;
	
	if ( frame ) {
		frame.open();
		return;
	}

	frame = wp.media.frames.wbte = wp.media({
		title: w_title,
		multiple: false,
		library: {
			type: 'image'
		},
		button: {
			text: w_button
		},
	});

	frame.on( 'select', function() {
		var attachment = frame.state().get('selection').first();
		var url        = attachment.attributes.url;
		jQuery( '#product_img_' + id ).val( attachment.attributes.id );
		jQuery( '#product_img_' + id ).closest('td').find('img').attr( 'src', url );
		jQuery( '#product_img_' + id ).closest('tr').find('input[name="id"]').prop( 'checked', true );
	});

	frame.open();

}
window.openMediaLib = function(product_id){
	openMediaLib(product_id);
}

function openMediaLibAll(productId) {
	event.preventDefault();
    
    var id       = productId;
    var w_title  = wbte_object.media_lib_title; 
    var w_button = wbte_object.media_lib_button; 

    // Initialize the media frame.
    var frame = wp.media.frames.wbte = wp.media({
        title: w_title,
        multiple: true,
        library: {
            type: 'image'
        },
        button: {
            text: w_button
        },
    });

	var imageIds = $('#product_img_ids_' + id).val();

	 // Preselect images when the frame is opened
	 if ( imageIds ) {
		imageIds = imageIds.split(',');
		frame.on('open', function() {
			var selection = frame.state().get('selection');
			
			imageIds.forEach(function(id) {
				var attachment = wp.media.attachment(id);
				attachment.fetch();
				selection.add(attachment ? [attachment] : []);
			});
		});
	}

    frame.on('select', function() {
		
		var attachments = frame.state().get('selection').toJSON();
    
		// Clear the container before appending to avoid duplicates
		$('#images_container_' + id).empty();
	
		// Reset imageIds to reflect the new selection state
		imageIds = [];
	
		// Process each selected image
		attachments.forEach(function(attachment) {
			var url = attachment.url; 
			$('#images_container_' + id).append('<img src="' + url + '" class="wbte-thumb-cust" />');
	
			// Add the attachment ID to the imageIds array
			imageIds.push(attachment.id.toString());
		});
	
		// Update the hidden input field with the new, joined list of image IDs
		$('#product_img_ids_' + id).val(imageIds.join(','));

        //Edit is done
        $('#product_img_' + id).closest('tr').find('input[name="id"]').prop('checked', true);
    });

    frame.open();
}
window.openMediaLibAll = function(product_id){
	openMediaLibAll(product_id);
}

function wbteBulkSetRowImages() {

	//Get the JSON content
	var jsonString  = $('#wbte_img_gallery').val();
	var attachments = JSON.parse(jsonString);

	if ( attachments.length > 0 ) {
		//Start progressbar
		$("#wbte_progressbar").progressbar("option", "value", false).show(); 

		$(tbl_cat + ' tbody tr:visible').each(function(){
			
			var form     = this.children[0].children[0];
			var id       = form['id'].value;
			var imageIds = new Set($('#product_img_ids_' + id).val() ? $('#product_img_ids_' + id).val().split(',') : []);

			// Object to track which URLs have already been appended to avoid duplicates
			var appendedUrls = {};
		
			attachments.forEach(function(attachment) {
				var url = attachment.url;
				var attachmentIdString = attachment.id.toString();
		
				// Check if the image URL has already been appended and if the image ID is unique
				if (!appendedUrls[url] && !imageIds.has(attachmentIdString)) {
					$('#images_container_' + id).append('<img src="' + url + '" class="wbte-thumb-cust" />');
					appendedUrls[url] = true; // Mark this URL as appended
					
					// Add the unique attachment ID to the Set
					imageIds.add(attachmentIdString);
				}
			});
		
			// Convert the Set back to an array and join it to update the hidden input field
			$('#product_img_ids_' + id).val(Array.from(imageIds).join(','));

			//Edit is done
			form['id'].checked = true;

		});

		//End progressbar
		$("#wbte_progressbar").progressbar("option", "value", 0).hide();
	}


}
window.wbteBulkSetRowImages = wbteBulkSetRowImages;


function openMediaLibAllBulk() {
	event.preventDefault();

    var w_title  = wbte_object.media_lib_title; 

    // Initialize the media frame.
    var frame = wp.media({
        title: w_title,
        multiple: true,
        library: {
            type: 'image'
        },
    });

    frame.on('select', function() {

		var main_attachments = frame.state().get('selection').toJSON();

		//Make JSON
		var jsonString = JSON.stringify(main_attachments);
		$('#wbte_img_gallery').val(jsonString).trigger('bulkSetEvent');

    });

    frame.open();
	
}
window.openMediaLibAllBulk = openMediaLibAllBulk;

function wbteDeleteAllImagesBulk(){
	$(cur_tbl + ' tbody tr:visible').each(function(){
		var id = this.children[0].id;
		if ($('#product_img_ids_'+id).val()){
			wbteDeleteAllImages(id);
		}
	});
}
window.wbteDeleteAllImagesBulk = wbteDeleteAllImagesBulk;

function wbteDeleteAllImages(id){
	$('#product_img_ids_'+id).val('');
	$('#images_container_'+id).empty();
	$('#images_container_' + id).closest('tr').find('input[name="id"]').prop( 'checked', true );
}
window.wbteDeleteAllImages = function(id){
	wbteDeleteAllImages(id);
}

function wbteRemoveThumbnail( id ) {
	$( '#product_img_' + id ).val( '' );
	$( '#product_img_' + id ).closest('td').find('img').attr( 'src', '' );
	$( '#product_img_' + id ).closest('tr').find('input[name="id"]').prop( 'checked', true );
	event.preventDefault();
}
window.wbteRemoveThumbnail = function(id){
	wbteRemoveThumbnail(id);
}

function wbteBulkRemoveImages() {
	
	$( cur_tbl + ' tbody tr:visible' ).each(
		function() {
			var form = this.children[0].children[0];
			var id   = form['product_img'].id;
			
			$('#'+id).val('');
			$('#'+id).closest('td').find('img').attr( 'src', '' );
			$('#'+id).closest('tr').find('input[name="id"]').prop( 'checked', true );
			
		}
	);

}
window.wbteBulkRemoveImages = wbteBulkRemoveImages;

//featured image
function wbteBulkSetImages() {
	
	event.preventDefault();
	var frame;
	var txtTitle  = wbte_object.media_lib_title;
	var txtButton = wbte_object.media_lib_button;
	
	if ( frame ) {
		frame.open();
		return;
	}

	frame = wp.media.frames.wbtebulk = wp.media({
		title: txtTitle,
		multiple: false,
		library: {
			type: 'image'
		},
		button: {
			text: txtButton
		},
	});

	frame.on( 'select', function() {
		var attachment = frame.state().get('selection').first();
		var url        = attachment.attributes.url;
		$( cur_tbl + ' tbody tr:visible' ).each( 
			function() {
				var form = this.children[0].children[0];
				var id   = form['product_img'].id;
				
				$('#'+id).val(attachment.attributes.id);
				$('#'+id).closest('td').find('img').attr( 'src', url );
				$('#'+id).closest('tr').find('input[name="id"]').prop( 'checked', true );
				
			}
		);
	});

	frame.open();
}
window.wbteBulkSetImages = wbteBulkSetImages;

function wbteChangeSearchUrl() {

	var option = $('#sales_filter option:selected').val();

	$('#wbte-pages').find('a').each(function(){
		var url    = $(this).attr('href');
		var newUrl = url.replace( /\&sales_filter\=.*/i, '' );
		$(this).attr( 'href', newUrl + '&sales_filter=' + option );
	});

	//Set filter for category dropdown
	$('#wbte-category-filter-home').find('input[name="sales_filter"]').val(option); //wbte-cat-select OLD ------------

}
window.wbteChangeSearchUrl = wbteChangeSearchUrl;

//Filter sales, tags and stock status
function wbteFilterSales() {

	var option = $('#sales_filter option:selected').val();
	var view   = $( '#form-gs-view' ).val();
	var table  = tbl;

	switch( view ) {
		case 'ext':
			table = tbl_ext;
			break;
		case 'custom':
			table = tbl_custom;
			break;
		case 'cat':
			table = tbl_cat;
			break;
	}

	//Set sales filter (in DOM)
	window.localStorage.setItem('sales_filter', option);
	
	var tags_arr = [];
	$('#tags-filter option:selected').each(function(){
		if ( '' !== this.value ) {
			tags_arr.push( this.value );
		}
	});
	window.localStorage.setItem('cas_tags_filter', tags_arr.toString());

	var brands_arr = [];
	if ( 'yes' === wbte_object.brand_filter ) {
		$('#brands-filter option:selected').each(function(){
			if ( '' !== this.value ) {
				brands_arr.push( this.value );
			}
		});
		window.localStorage.setItem('cas_brands_filter', brands_arr.toString());
	}

	var stock_arr = [];
	$('#stock-filter option:selected').each(function(){
		if ( '' !== this.value ){
			stock_arr.push( this.value );
		}
	});
	window.localStorage.setItem('cas_stock_filter', stock_arr.toString());

	$( table + ' tbody tr' ).each(
		function() {
			var form = this.children[0].children[0];
			var sale = form['saleprice'].value;

			if ( 'onSale' === option ) {
				if ( sale && sale > 0 ) {
					$(this).show();
				} else {
					$(this).hide();
				}
			} else if ( 'noSale' === option ) {
				if ( sale === '' ) {
					$(this).show();
				} else {
					$(this).hide();
				}
			} else {
				$(this).show();
			}

		}
	);

	var p_arr  = [];
	var selVal = $('#cas-range-type option:selected').val();
	
	$( table + ' tbody tr:visible' ).each(function() {
		
		var form              = this.children[0].children[0];
		var tags              = form['tags'].value;
		var brands            = form['brands'].value;
		var status            = form['stock_status'].value;
		var showBasedOnTags   = true;
		var showBasedOnBrands = true;
		var showBasedOnStatus = true;
	
		// Tags
		if (tags_arr.length > 0) {
			showBasedOnTags = tags_arr.some(tag => tags.includes(tag));
		}
	
		// Brands
		if (brands_arr.length > 0) {
			showBasedOnBrands = brands_arr.some(brand => brands.includes(brand));
		}

		// Stock
		if ( stock_arr.length > 0 ) {
			showBasedOnStatus = stock_arr.some(s => status.includes(s) );
		}
	
		// Show / hide
		if ( showBasedOnTags && showBasedOnBrands && showBasedOnStatus ) {
			$(this).show();
			if ('prod' === view) {
				p_arr.push(parseFloat(form[selVal].value || 0));
			}
		} else {
			$(this).hide();
		}
	});
	
	wbteChangeSearchUrl();

	//Update Range filter
	if ( 'prod' === view ) {
		casSliderUpdater( Math.min(...p_arr), Math.max(...p_arr) );
	}

}
window.wbteFilterSales = wbteFilterSales;

function wbte_reset_filters() {

	//Reset product tags filter
	$('#product_tag').multiselect('uncheckAll');
	$('#product_tag').multiselect('refresh');
	$('#product_tag').val('').change();
	window.localStorage.setItem('cas_tags_filter', '');

	//Reset brands filter
	if ( 'yes' === wbte_object.brand_filter ) {
		$('#product_brand').multiselect('uncheckAll');
		$('#product_brand').multiselect('refresh');
		$('#product_brand').val('').change();
		window.localStorage.setItem('cas_brands_filter', '');
	}

	//Reset sales filter
	$('#sales_filter').multiselect('uncheckAll');
	$('#sales_filter').multiselect('refresh');
	$('#sales_filter').val('').change();
	window.localStorage.setItem('sales_filter', '');

	//Reset stock status filter
	$('#stock_status').multiselect('uncheckAll');
	$('#stock_status').multiselect('refresh');
	$('#stock_status').val('').change();
	window.localStorage.setItem('cas_stock_filter', '');

	//RESET SEARCH
	$('#product_search').val('').change();

	wbteChangeSearchUrl();
	
	//Range filter
	if ( 'prod' === $( '#form-gs-view' ).val() ) {
		$('#wbtetable tbody tr').show();
		casRangeTypeChanged( '', '' );
	}
}
window.wbte_reset_filters = wbte_reset_filters;

function wbteChangeTagsUrl() {

	var option = $('#tags-filter option:selected').val();

	$('#wbte-pages').find('a').each(function(){
		var url    = $(this).attr('href');
		var newUrl = url.replace( /\&tags-filter\=.*/i, '' );
		$(this).attr( 'href', newUrl + '&tags-filter=' + option );
	});

}
window.wbteChangeTagsUrl = wbteChangeTagsUrl;

function wbteChangeStockStatusUrl() {

	var option = $('#stock_status option:selected').val();

	$('#wbte-pages').find('a').each(function(){
		var url    = $(this).attr('href');
		var newUrl = url.replace( /\&stock_status\=.*/i, '' );
		$(this).attr( 'href', newUrl + '&stock_status=' + option );
	});

}
window.wbteChangeStockStatusUrl = wbteChangeStockStatusUrl;

function wbte_hide_desc(id) {
	event.preventDefault();
	$('#wbte_desc_'+id).removeClass('wbte-prod-desc').addClass('wbte-prod-desc-hide');
	$('#btn_desc_'+id).removeClass('btn-desc').addClass('btn-desc-hide');
	$('#wbte_ul_desc_'+id).removeClass('wbte-ul-info').addClass('wbte-ul-info-hide');
}
window.wbte_hide_desc = function(id){
	wbte_hide_desc(id);
}

function wbte_show_desc(id) {
	event.preventDefault();
	var is_visible = ( 'wbte-prod-desc' === $('#wbte_desc_'+id).prop('class') ) ? true : false;
	if ( ! is_visible ) {
		$('#wbte_desc_'+id).removeClass('wbte-prod-desc-hide').addClass('wbte-prod-desc');
		$('#btn_desc_'+id).removeClass('btn-desc-hide').addClass('btn-desc');
		$('#wbte_ul_desc_'+id).removeClass('wbte-ul-info-hide').addClass('wbte-ul-info');
		
		//Add WP Editor
		if ( wbte_object.disable_description !== 'yes' ){
			wp.editor.initialize( 'desc_' + id );
		}

	} else {
		$('#wbte_desc_'+id).removeClass('wbte-prod-desc').addClass('wbte-prod-desc-hide');
		$('#btn_desc_'+id).removeClass('btn-desc').addClass('btn-desc-hide');
		$('#wbte_ul_desc_'+id).removeClass('wbte-ul-info').addClass('wbte-ul-info-hide');
	}
}
window.wbte_show_desc = function(id){
	wbte_show_desc(id);
}

function wbte_show_hide_desc() {
	var is_checked = $('#wbte-chk-show-desc').prop('checked');
	if (is_checked) {
		$('table div .wbte-prod-desc-hide').removeClass('wbte-prod-desc-hide').addClass('wbte-prod-desc');
		$('table button[name="btn_desc"]').removeClass('btn-desc-hide').addClass('btn-desc');
		$('table ul').removeClass('wbte-ul-info-hide').addClass('wbte-ul-info');

		//Add WP Editor
		if ( wbte_object.disable_description !== 'yes' ){
			$('table tbody form').each( function() {
				var frm  = this;
				var pId  = frm['id'].value;
				wp.editor.initialize( 'desc_' + pId );
			});
		}

	} else {
		$('table div .wbte-prod-desc').removeClass('wbte-prod-desc').addClass('wbte-prod-desc-hide');
		$('table button[name="btn_desc"]').removeClass('btn-desc').addClass('btn-desc-hide');
		$('table ul').removeClass('wbte-ul-info').addClass('wbte-ul-info-hide');
	}
}
window.wbte_show_hide_desc = wbte_show_hide_desc;

function wbte_reset_table_values() {
	var view  = $( '#form-gs-view' ).val();
	var table = ( 'ext' === view ) ? tbl_ext : tbl;

	if ( 'custom' === view ) {
		table = tbl_custom;
	}

	if ( 'cat' === view ) {
		reloadPage(200);
	}

	$( table + ' tbody form input[name="id"]:checked' ).each( function () {
		$(this.form)[0].reset();
		if ( 'prod' === view ) { 
			this.form['totcol'].value = ( this.form['stock'].value > 0 && this.form['price'].value > 0 ) ? ( parseFloat( this.form['stock'].value ) * parseFloat( this.form['price'].value ) ).toFixed(2) : '';
		}
	});

	$( '#saveall' ).css( 'color', '' );
	$( '#saveall-foot' ).css( 'color', '' );
}
window.wbte_reset_table_values = wbte_reset_table_values;

//Remove single brand from product
function wbteRemoveBrand(termId, productId) {
	var nonce_val = jQuery( '#_wpnonce' ).val();
	var json_data = { action: 'wbte_update_product_brands_data', nonce: nonce_val, product_id: productId, term_id: termId };

	$.ajax(
		{
			type: 'POST',
			url: ajaxurl,
			datatype: 'json',
			data: json_data,
			success: function( response ) {
				//console.log( response );
				jQuery('#link_'+termId+'_'+productId).remove();
			},
			error: function( response ){
				console.log( response );
			}
		}
	);
}
window.wbteRemoveBrand = function(termId, productId) {
	wbteRemoveBrand(termId,productId);
}

//--------- Linked product tab -----------------------------------------------------------------

function cas_upsell_get_category( obj, select_name, categories ) {
	event.preventDefault();
	var form                  = $( '#' + obj.id )[0];
	var selected              = $( '#wbte-category-filter').val();
	form[categories].value    = selected.filter(n => n).toString();
	form['categories2'].value = $("#cas-form-category2-filter input[name='categories2']").val();
	form[select_name].remove();
	form.submit();
}
window.cas_upsell_get_category = function(obj, select_name, categories){
	cas_upsell_get_category( obj, select_name, categories );
}

function cas_upsell_products_get( obj, select_name, categories ) {
	event.preventDefault();
	var form                 = $( '#' + obj.id )[0];
	var selected             = $( '#wbte-category-filter-upsell').val();
	var categories1          = $("#cas-form-category-filter input[name='categories']").val();
	form['categories'].value = categories1;
	form[categories].value   = selected.filter(n => n).toString();
	form[select_name].remove();
	form.submit();
}
window.cas_upsell_products_get = function(obj, select_name, categories){
	cas_upsell_products_get(obj, select_name, categories);
}

function wbteLinkProducts( type ) {

	var nonce_val  = $( '#_wpnonce' ).val();
	var checkedIds = [];
	var checkedUp  = [];
	
	$('#wbte-linked-product-table tbody tr input[name="productId"]:checked').each(function() {
		checkedIds.push($(this).prop('id'));
	});
	
	$('#wbte-up-cross-sell-table tbody tr input[name="productUp"]:checked').each(function() {
		checkedUp.push($(this).prop('id'));
	});
	
	//Make sure we have checked products in both tables
	if ( checkedIds.length > 0 && checkedUp.length > 0 ) {

		var json_data = { action: 'wbte_upsell_set_linked_products', nonce: nonce_val, linktype: type, products: checkedIds, upsells: checkedUp };

		jQuery.ajax(
			{
				type: 'POST',
				url: ajaxurl,
				datatype: 'json',
				data: json_data,
				success: function( response ) {
					//console.log( response );
					reloadPage('200');
				},
				error: function( response ){
					console.log( response );
				}
			}
		);

	}
}
window.wbteLinkProducts = function(type){
	wbteLinkProducts(type);
}

function wbteCheckAllRows( table, inputName, obj ) {

	$('#'+table+' tbody tr:visible').each( function() {
		var input = $(this).find('input[name="'+inputName+'"]');
		input.prop('checked', !input.prop('checked') && obj.checked);
		$(this).toggleClass('cas-selected-row', input.prop('checked'));
	});

}
window.wbteCheckAllRows = function(table, inputName, obj){
	wbteCheckAllRows(table, inputName, obj);
}

function wbteRemoveCrossProduct( id, upsellId, type ) {
	var nonce_val = $( '#_wpnonce' ).val();
	var json_data = { action: 'wbte_upsell_remove_linked_product', nonce: nonce_val, linktype: type, product: id, upsell: upsellId };

	$('#'+type+'_'+id+'_'+upsellId).remove();

	jQuery.ajax(
		{
			type: 'POST',
			url: ajaxurl,
			datatype: 'json',
			data: json_data,
			success: function( response ) {
				//console.log( response );
			},
			error: function( response ){
				console.log( response );
			}
		}
	);
}
window.wbteRemoveCrossProduct = function(id, upsellId, type){
	wbteRemoveCrossProduct(id, upsellId, type);
}

function wbte_SearchSubmit( obj ) {
	var is_db = ( $( '#cas-search-toggle' ).prop('checked') ) ? 'yes' : 'no';
	var input = $( '#wbte_s' ).val();
	var form  = $( '#' + obj.id )[0];
	
	if ('yes' === is_db ){
		form['categories2'].value = $("#cas-form-category2-filter input[name='categories2']").val();
	}

	switch( is_db ) {
		case 'no':
			event.preventDefault();
			( '' === input ) ? obj.submit() : wbte_filterCrossRows('wbte-linked-product-table', input); 
			break;
		default:
			obj.submit();
			break;
	}
}
window.wbte_SearchSubmit = function(obj){
	wbte_SearchSubmit(obj);
}

function wbte_SearchSubmit2( obj ) {
	var is_db = ( $( '#cas-search-toggle2' ).prop('checked') ) ? 'yes' : 'no';
	var input = $( '#wbte_s2' ).val();
	var form  = $( '#' + obj.id )[0];
	
	if ('yes' === is_db ){
		form['categories'].value = $("#cas-form-category-filter input[name='categories']").val();
	}

	switch( is_db ) {
		case 'no':
			event.preventDefault();
			( '' === input ) ? obj.submit() : wbte_filterCrossRows('wbte-up-cross-sell-table', input); 
			break;
		default:
			obj.submit();
			break;
	}
}
window.wbte_SearchSubmit2 = function(obj){
	wbte_SearchSubmit2(obj);
}

function wbte_filterCrossRows( table, input ) {

	$('#'+table+' tbody tr').each( function(){
		if (undefined === this.children[3]){
			return;
		} 
		var name = this.children[3].children[0].innerText;
	
		if ( input.length > 0 ) {
			if( name.toLowerCase().indexOf( input.toLowerCase() ) !== -1 ) {
				$(this).show();
			} else {
				$(this).hide();
			}
		} else {
			$(this).show();
		}
	});
}
window.wbte_filterCrossRows = function(table,input){
	wbte_filterCrossRows(table,input);
}

// ------- Attributes & Custom Fields tab ----------
function wbte_bulk_add_attributes() {

	//wbte_attributes_action
	var action     = $('#wbte_attributes_action option:selected').val(); 
	var attributes = [];

	//Reset
	$('#wbte_attributes_action').val('');

	//Get selected attributes
	$('#bulk_add_attribute option:selected').each(function() {
		if ( '' !== this.value ) {
			attributes.push(this.value.toLowerCase());
		}
	});

	$('#wbtetable_custom tbody tr:visible').each(function(){
		var form         = this.children[0].children[0];
		var prod_id      = form['id'].value;
		var is_variation = ( form['attributes'].dataset.isVariation === '1' ) ? true : false;

		if (is_variation) {
			return;
		}

		var prod_attr       = form['attributes'].value.length > 0 ? form['attributes'].value.split(';') : [];
		var prod_attr_lower = prod_attr.map( function(attr) {
            return attr.toLowerCase();
        });
		
        switch ( action ) {
            case 'add':
                attributes.forEach(function(attr) {
                    if (!prod_attr_lower.includes(attr)) {
                        prod_attr.push(attr);
                        prod_attr_lower.push(attr);
                    }
                });
                break;
            case 'set':
                prod_attr       = attributes.slice();
                prod_attr_lower = attributes.slice();
                break;
            case 'remove':
                attributes.forEach(function(attr) {
                    var index = prod_attr_lower.indexOf(attr);
                    if (index > -1) {
                        prod_attr.splice(index, 1);
                        prod_attr_lower.splice(index, 1);
                    }
                });
                break;
            case 'clear':
                prod_attr       = [];
                prod_attr_lower = [];
                break;
        }

		// Remove empty values and sort
		var clean_arr = prod_attr.filter(e => Object.keys(e).length > 0).sort();

		form['id'].checked       = true;
		form['attributes'].value = clean_arr.join(';');
		
		var div_string = '';
		for( i = 0; i < clean_arr.length; i++){
			var txt_arr = clean_arr[i].split('=');
			div_string += txt_arr[0].charAt(0).toUpperCase() + txt_arr[0].slice(1) + ': ' + txt_arr[1] + '<br/>'; 
		}
		$('#'+prod_id+'_attributes_txt').html(div_string);

	});

}
window.wbte_bulk_add_attributes = wbte_bulk_add_attributes;

function wbteBulkCustomValues(custom_id) {

	var in_val   = $( '#' + custom_id + '_value' ).val();
	var in_type  = $( '#' + custom_id + '_select option:selected' ).val();
	
	$( tbl_custom + ' tbody tr:visible' ).each(
		function() {
			
			var form = this.children[0].children[0];
			
			switch(in_type) {
				case 'add':
					form[custom_id].value += in_val;
					break;
				case 'set':
					form[custom_id].value = in_val;
					break;
				case 'remove':
					var regex = new RegExp(in_val, 'g');
					form[custom_id].value = form[custom_id].value.replace(regex, '');
					break;
				case 'clear':
					form[custom_id].value = '';
					break;
				case 'append':
					form[custom_id].value += ' ' + in_val; // Optional separator
					break;
				case 'prepend':
					form[custom_id].value = in_val + ' ' + form[custom_id].value; // Optional separator
					break;
				case 'increment':
					form[custom_id].value = parseFloat(form[custom_id].value) + parseFloat(in_val);
					break;
				case 'decrement':
					form[custom_id].value = parseFloat(form[custom_id].value) - parseFloat(in_val);
					break;
				case 'uppercase':
					form[custom_id].value = form[custom_id].value.toUpperCase();
					break;
				case 'lowercase':
					form[custom_id].value = form[custom_id].value.toLowerCase();
					break;
				case 'capitalize':
					form[custom_id].value = form[custom_id].value.replace(/\b\w/g, function(char) {
						return char.toUpperCase();
					});
					break;
			}

			form['id'].checked = true;
			
		}
	);

}
window.wbteBulkCustomValues = function(id){
	wbteBulkCustomValues(id);
}

//Save all data from custom table
function wbteSaveCustomTable() {

	var nonce_val = $( '#_wpnonce' ).val();
	var json_data = { action: 'wbte_save_attributes_and_custom', nonce: nonce_val, rows: [] };
	var customs   = wbte_object.custom_fields;

	$( tbl_custom + ' tbody form input[name="id"]:checked' ).each( function() {
		
		var frm  = this.form;
		
		var frm_data = { 
			id: frm['id'].value,
			name: frm['product_name'].value, 
			name_update: frm['name_update'].value,
			parent_id: frm['parent_id'].value,
			attributes: frm['attributes'].value, 
			custom_fields: {},
		};

		for ( i = 0; i < customs.length; i++ ) {
			frm_data.custom_fields[customs[i].slug] = frm[customs[i].slug].value;
		}
		
		json_data.rows.push( frm_data );
		
		if ( json_data.rows.length === 1 ){
			progressBar('start');
		}
	});

	if ( json_data.rows.length > 0 ) {

		$.ajax(
			{
				type: 'POST',
				url: ajaxurl,
				datatype: 'json',
				data: json_data,
				success: function( response ) {
					console.log( response );
					progressBar('stop');
					reloadPage(300);
				},
				error: function( response ){
					progressBar('stop');
					console.log( response );
				}
			}
		);
	}
	
	event.preventDefault();

}
window.wbteSaveCustomTable = wbteSaveCustomTable;

})(jQuery);