<?php
/**
 * Bulk Table Editor custom 
 *
 * @package BulkTableEditor/includes
 */

defined( 'ABSPATH' ) || exit;

require_once __DIR__ . '/class-wbte-functions.php';

const CAS_WBTE_CATEGORY_IMAGES_TABLE = 'Wbte_Category_Images_Table';

/**
 * Custom field class
 */
class Wbte_Category_Images_Table {

	/**
	 * Get table and its data
	 */
	public static function wbte_get_custom_table() {

		$functions     = new WbteFunctions();
		$json_products = $functions->wbte_get_json_products();
		$page_num      = ( ! empty( filter_input( 1, 'paged', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'paged', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : 1;
		$view          = filter_input( 1, 'view', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$products      = (array) $json_products['rows'][0];
		$total_pages   = $json_products['num-pages'];

		?>
		
		<div class="display-desktop"> 
			<div class="cas-action-box" style="width:99%;margin-right:20px;">
				<div style="text-align:right;padding-right:10px;">
					<button type="button" id="save_cat_img" class="button action" onclick='saveCatImgAll();' style="width:150px;"><i class="fas fa-database"></i> <?php esc_html_e( 'Save Changes', 'woo-bulk-table-editor' ); ?></button>
				</div>
			</div>
			<!-- Table -->
			<?php wp_nonce_field( 'wbte_id' ); ?>
			<input type="hidden" id="wbte_img_gallery" value="" oninput="wbteBulkSetRowImages();">
			<table class="wp-list-table widefat fixed striped posts" id="wbtetable_cat" style="width:100%;">
				<thead>
					<?php self::wbte_get_table_head( $functions ); ?>
				</thead>
				<tbody id="the-list"> 
					<?php  
					foreach ( $products as $product ) {
						$variations = ( is_a( $product, 'WC_Product' ) && ( $product->is_type( 'variable' ) ) ) ? $product->get_available_variations() : '';
						$variations = ( is_a( $product, 'WC_Product_Subscription' ) && ( $product->is_type( 'variable-subscription' ) ) ) ? $product->get_available_variations() : $variations; 

						if ( ! empty( $variations ) ) {
							foreach ( $variations as $variation ) {
								$variation_obj = wc_get_product( $variation['variation_id'] );
								self::wbte_get_table_row( $variation_obj, true, $functions );
							}
						} else {
							self::wbte_get_table_row( $product, false, $functions );
						}
					};
					?>
				</tbody>
				<tfoot>
					<tr>
						<td colspan="8"> </td>
					</tr>
				</tfoot>
			</table>
			
			<div id="wbte-paging-bottom2" class="cas-div-paging" style="margin-top: 6px;">
				<span class="cas-pagination">
				<?php
				
				$args = array( // WPCS: XSS ok.
					'base'      => '%_%',
					'format'    => '?paged=%#%',
					'current'   => $page_num,
					'total'     => $total_pages,
					'prev_text' => is_rtl() ? '&rarr;' : '&larr;',
					'next_text' => is_rtl() ? '&larr;' : '&rarr;',
					'type'      => 'plain',
					'end_size'  => 1,
					'mid_size'  => 1,
				);
				echo wp_kses( paginate_links( $args ), 'post' );
				
				?>
				</span>
			</div>

			</div>
		<div>
			<div id="wbte_select_single_categories_div" style="display:none;" title="<?php esc_html_e( 'Select Categories', 'woo-bulk-table-editor' ); ?>">
				<?php self::wbte_get_categories_for_bulk_select( 'wbte_select_single_categories' ); ?><br/>
				<select class="input-select" id="bulkcat_select_single_type" style="max-width:370px;">
					<?php $functions->wbte_get_bulk_options( 'bulkcat2' ); ?>
				</select>
				<p>
					<button id="wbte_cancelCategory" class="button"><?php esc_html_e( 'Cancel', 'woo-bulk-table-editor' ); ?></button>
				</p>
			</div>
		<!-- FINAL END DIV -->
		</div>		
		<?php
	}

	/**
	 * Get table head
	 */
	public static function wbte_get_table_head( $functions ) {

		?>
		<tr>
			<th class="th-first" style="width:20px;"><input type="checkbox" id="checkall" onclick="checkVisible();"></th>
			<th style="width:25%;">
				<a href="#" onclick="sort(1,'text');"><?php esc_html_e( 'Name', 'woo-bulk-table-editor' ); ?></a> <i id="s1" class="fas fa-sort" style="padding-right: 10px;"></i>
			</th>
			<th style="width:20px;"></th>
			<th style="width:60px;text-align:center;"><i class="fa-regular fa-image" title="<?php esc_html_e( 'Featured Image', 'woo-bulk-table-editor' ); ?>"></i></th>
			<th style="width:25%;text-align:left;">
				<i class="fa-regular fa-images"></i> <?php esc_html_e( 'Product Gallery', 'woo-bulk-table-editor' ); ?> 
				<i class="fa-solid fa-circle-info" title="<?php esc_html_e( 'Variants share the same images. Changing images for one variation applies to all.', 'woo-bulk-table-editor' ); ?>"></i> 
			</th>
			<th style="width:70px;"></th>
			<th style="width:25%;">
				<?php esc_html_e( 'Categories', 'woo-bulk-table-editor' ); ?> 
				<i class="fa-solid fa-circle-info" title="<?php esc_html_e( 'Variants share the same categories. Changing categories for one variation applies to all.', 'woo-bulk-table-editor' ); ?>"></i> 
			</th>
			<th></th>
		</tr>
		
		<tr class="th-info">
			<th></th>
			<th style="vertical-align: middle;">
				<strong><?php esc_html_e( 'Bulk Editor', 'woo-bulk-table-editor' ); ?></strong><br />
				<span class="th-desc"><?php esc_html_e( 'Enter values (input) then select bulk function.', 'woo-bulk-table-editor' ); ?>
				</span>
			</th>
			<th class="wbte_td_border"></th>
			<th class="wbte_td_border" style="vertical-align:bottom;text-align:right;">
				<i class="fas fa-edit" style="padding-right:3px;font-size:large;color:#2271b1;" onclick="wbteBulkSetImages();" title="<?php esc_html_e( 'Bulk Set Images', 'woo-bulk-table-editor' ); ?>"></i>
				<i class="fas fa-trash" style="font-size:large;color:#2271b1;padding-right:3px;" onclick="wbteBulkRemoveImages();" title="<?php esc_html_e( 'Bulk Remove Images', 'woo-bulk-table-editor' ); ?>"></i> 
			</th>
			<th style="vertical-align:bottom;">
				<div id="wbte_progressbar"></div>
			</th>
			<th class="th-center wbte_td_border" style="vertical-align:bottom;">
				<i class="fas fa-edit" style="padding-right:3px;font-size:large;color:#2271b1;" onclick="openMediaLibAllBulk();"  title="<?php esc_html_e( 'Bulk Set Images', 'woo-bulk-table-editor' ); ?>"></i>
				<i class="fa-solid fa-eye" style="padding-right:3px;font-size:large;color:#2271b1;" onclick="wbteZoomImagesBulk();" title="<?php esc_html_e( 'Bulk Zoom Images', 'woo-bulk-table-editor' ); ?>"></i>
				<i class="fas fa-trash" style="font-size:large;color:#2271b1;" onclick="wbteDeleteAllImagesBulk();" title="<?php esc_html_e( 'Bulk Remove Images', 'woo-bulk-table-editor' ); ?>"></i> 
			</th>
			<th>
				<!-- CATEGORY -->
				<?php self::wbte_get_categories_for_bulk_select(); ?><br/>
				<select class="input-select" name="bulkcat_select_type" id="bulkcat_select_type" onchange="bulkSetValues('bulkcat_select','');">
					<?php $functions->wbte_get_bulk_options( 'bulkcat' ); ?>
				</select>
			</th>
			<th style="vertical-align: top;">
			</th>
		</tr>
		<?php
	}

	/**
	 * Get table rows
	 */
	public static function wbte_get_table_row( $product, $is_variable, $functions ) {

		$product_name   = $product->get_name();
		$attributes     = $product->get_attributes();
		$product_id     = $product->get_id();
		$type           = ( $is_variable ) ? 'var' : 'prod';
		$tag_ids        = $product->get_tag_ids();
		$attachment_ids = $product->get_gallery_image_ids();

		if ( $is_variable ) {
			$product_id     = $product->get_parent_id();
			$parent         = wc_get_product( $product_id );
			$tag_ids        = $parent->get_tag_ids();
			$attachment_ids = $parent->get_gallery_image_ids();
		}
		
		if ( ! empty( $attributes ) && is_array( $attributes ) ) {
			foreach ( $attributes as $key => $val ) {
				if ( isset( $val ) && is_string( $val ) && strlen( $val ) > 0 ) {
					$product_name .= ', ' . ucfirst( $val );
				}
			}
		}

		$tags = $functions->wbte_get_product_tags( $tag_ids );
		
		?>
		
		<tr class="lozad" style="vertical-align: top;" data-type="<?php echo esc_attr( $type ); ?>">
			<td id="<?php echo esc_attr( $product->get_id() ); ?>" class="cas-td-chk">
				<form id="frm_<?php echo esc_attr( $product->get_id() ); ?>_<?php echo esc_attr( wp_rand( 1, 50000 ) ); ?>" method="post">
				<input name="id" value="<?php echo esc_attr( $product->get_id() ); ?>" type="checkbox">
				<input type="hidden" name="saleprice" value="<?php echo esc_attr( $product->get_sale_price() ); ?>">
				<input type="hidden" name="tags" value="<?php echo esc_attr( $tags ); ?>">
				<input type="hidden" name="stock_status" value="<?php echo esc_attr( $product->get_stock_status() ); ?>">
				<input type="hidden" name="brands" value="<?php echo esc_attr( $functions->wbte_get_product_brands( $product_id, 'slug' ) ); ?>">
				<input type="hidden" name="parent_id" value="<?php echo esc_attr( $is_variable ? $product_id : '' ); ?>">
			</td>
			<td>
				<?php
				if ( ! $is_variable ) {
					?>
					<input type="text" class="input-txt-name" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="true">
					<?php
				} else {
					?>
					<input type="hidden" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="false">
					<?php echo esc_attr( $product_name ); ?>
					<?php
				}
				?>
			</td>
			<td class="wbte_td_border">
				<a href="<?php echo esc_url( get_admin_url() . 'post.php?post=' . $product_id . '&action=edit' ); ?>" target="_new" title="<?php esc_attr_e( 'Edit product', 'woo-bulk-table-editor' ); ?>">
					<i class="fas fa-edit"></i>
				</a>
			</td>
			<td class="wbte_td_border">
				<!-- FEATURED -->
				<input type="hidden" id="product_img_<?php echo esc_attr( $product->get_id() ); ?>" name="product_img" value="<?php echo esc_attr( $product->get_image_id() ); ?>">
				<a href="#" onclick="openMediaLib('<?php echo esc_js( $product->get_id() ); ?>')" title="<?php esc_html_e( 'Click to add or edit', 'woo-bulk-table-editor' ); ?>">
				<?php
				if ( $product->get_image_id() ) {
					if ( ! $is_variable ) {
						echo wp_kses( get_the_post_thumbnail( $product_id, array( '28', '28' ), array( 'class' => 'wbte-thumb-cust' ) ), $functions->wbte_get_allowed_html() );
					} else {
						$variation = new WC_Product_Variation( $product->get_id() );
						echo wp_kses( $variation->get_image( array( '28', '28' ), array( 'class' => 'wbte-thumb-cust' ) ), $functions->wbte_get_allowed_html() );
					}
				} else {
					?>
					<img height="28" src="<?php echo esc_attr( wc_placeholder_img_src() ); ?>" class="wbte-thumb-cust wp-post-image" >
					<?php
				}
				?>
				</a>
				
				<a href="#" onclick="wbteRemoveThumbnail( <?php echo esc_js( $product->get_id() ); ?> );" title="<?php esc_html_e( 'Click to remove', 'woo-bulk-table-editor' ); ?>"> <i class="fas fa-trash" style="float:right;padding-right:3px;font-size:medium;padding-top:9px;"></i> </a>
			</td>
			<td>
				<!-- All images -->
				<input type="hidden" id="product_img_ids_<?php echo esc_attr( $product->get_id() ); ?>" name="product_img_ids" value="<?php echo esc_attr( implode( ',', $attachment_ids ) ); ?>">
				<div id="images_container_<?php echo esc_attr( $product->get_id() ); ?>" onclick="openMediaLibAll('<?php echo esc_js( $product->get_id() ); ?>');" title="<?php esc_html_e( 'Set Images', 'woo-bulk-table-editor' ); ?>">
					<?php foreach ( $attachment_ids as $img_id ) : ?>
						<img src="<?php echo wp_kses( wp_get_attachment_image_url( $img_id, 'thumbnail' ), 'post' ); ?>" class="wbte-thumb-cust">
					<?php endforeach; ?>
				</div>
			</td>
			<td class="th-center wbte_td_border">
				<i class="fas fa-edit" style="padding-right:3px;font-size:medium;color:#2271b1;" onclick="openMediaLibAll('<?php echo esc_js( $product->get_id() ); ?>');"  title="<?php esc_html_e( 'Set Images', 'woo-bulk-table-editor' ); ?>"></i>
				<i class="fa-solid fa-eye" style="padding-right:3px;font-size:medium;color:#2271b1;" onclick="wbteZoomImages('<?php echo esc_js( $product->get_id() ); ?>');" title="<?php esc_html_e( 'Zoom Images', 'woo-bulk-table-editor' ); ?>"></i>
				<i class="fas fa-trash" style="font-size:medium;color:#2271b1;" onclick="wbteDeleteAllImages('<?php echo esc_js( $product->get_id() ); ?>');" title="<?php esc_html_e( 'Remove Images', 'woo-bulk-table-editor' ); ?>"></i> 
			</td>
			<td>
				<!-- Categories -->
				<div class="cas-td-scroll" id="category_list_<?php echo esc_attr( $product->get_id() ); ?>" title="<?php esc_attr_e( 'Click to delete', 'woo-bulk-table-editor' ); ?>">
				<?php
				$_categories = get_the_terms( $product_id, 'product_cat' );
				$_cat_ids    = array();

				if ( $_categories ) {
					$_cat_list = '';
					foreach ( $_categories as $_cat ) {
						array_push( $_cat_ids, $_cat->term_id );
						$del_id     = $_cat->term_id . '_' . $product->get_id();
						$_cat_list .= '<div id="link_' . $del_id . '"><a href="#" onclick="wbteRemoveCat(' . esc_js( $_cat->term_id ) . ',' . esc_js( $product->get_id() ) . ');"><i class="fas fa-check-square"></i> ' . esc_attr( $_cat->name ) . '</a></div>';
					}
					echo wp_kses( $_cat_list, $functions->wbte_get_allowed_html() );
				}
				?>
				</div>
				<input type="hidden" id="product_cat_ids_<?php echo esc_attr( $product->get_id() ); ?>" name="product_cat_ids" value="<?php echo esc_attr( implode( ',', $_cat_ids ) ); ?>">
			</td>
			<td class="th-center">
				<i class="fas fa-edit wbte-edit-single-cat" style="font-size:medium;color:#2271b1;" title="<?php esc_attr_e( 'Edit', 'woo-bulk-table-editor' ); ?>"></i>
				<i class="fas fa-trash" style="font-size:medium;color:#2271b1;" onclick="wbteDeleteCategories('<?php echo esc_js( $product->get_id() ); ?>');" title="<?php esc_attr_e( 'Remove categories (set default)', 'woo-bulk-table-editor' ); ?>"></i> 
				</form>
			</td>
		</tr>
		<?php
	}

	/**
	 * Get product categories - select box
	 *
	 */
	public static function wbte_get_categories_for_bulk_select( $name = '' ) {
		
		$options     = get_option( 'wbte_options' );
		$order_by    = ! empty( $options['wbte_categories_orderby'] ) ? $options['wbte_categories_orderby'] : 'menu_order';
		$select_name = ( empty( $name ) ) ? 'bulk_product_cat' : $name;

		$args = array(
			'pad_counts'         => 1,
			'show_count'         => 0,
			'hierarchical'       => 1,
			'hide_empty'         => 0,
			'show_uncategorized' => 1,
			'orderby'            => $order_by,
			'show_option_none'   => __( 'Select Categories', 'woo-bulk-table-editor' ),
			'option_none_value'  => '',
			'value_field'        => 'id',
			'taxonomy'           => 'product_cat',
			'name'               => $select_name,
			'class'              => 'dropdown_product_cat',
		);
		?>
		
		<?php
			wp_dropdown_categories( $args );
		?>

		<?php
	}

}

