<?php
/**
 * Bulk Table Editor custom 
 *
 * @package BulkTableEditor/includes
 */

defined( 'ABSPATH' ) || exit;

require_once __DIR__ . '/class-wbte-functions.php';

const CAS_WBTE_CUSTOM_FIELDS = 'Wbte_Custom_Fields';

/**
 * Custom field class
 */
class Wbte_Custom_Fields {

	/**
	 * Get all custom fields from settings
	 */
	public static function wbte_get_custom_fields_array() {

		$options       = get_option('wbte_options');
		$custom_fields = array();

		for ( $i = 1; $i <= 5; $i++ ) {
			
			$custom_field = new stdClass();
			$custom_field->name = isset( $options["wbte_custom_field_{$i}_name"] ) ? $options["wbte_custom_field_{$i}_name"] : '';
			$custom_field->type = isset( $options["wbte_custom_field_{$i}_type"] ) ? $options["wbte_custom_field_{$i}_type"] : '';
			$custom_field->slug = isset( $options["wbte_custom_field_{$i}_slug"] ) ? $options["wbte_custom_field_{$i}_slug"] : '';
			$custom_field->use  = isset( $options["wbte_use_custom_field_{$i}"] ) ? $options["wbte_use_custom_field_{$i}"] : '';

			if ( 'yes' === $custom_field->use ) {
				$custom_fields[] = $custom_field;
			}
		}

		return $custom_fields;
	}

	/**
	 * Get table and its data
	 */
	public static function wbte_get_custom_table() {

		$functions     = new WbteFunctions();
		$json_products = $functions->wbte_get_json_products();
		$page_num      = ( ! empty( filter_input( 1, 'paged', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'paged', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : 1;
		$view          = filter_input( 1, 'view', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$products      = (array) $json_products['rows'][0];
		$total_pages   = $json_products['num-pages'];
		
		?>
		<div class="display-desktop"> <!-- Table -->
			<div class="cas-action-box" style="width:99%;margin-right:20px;">
				<div style="text-align:right;padding-right:10px;">
					<button class="button" id="reset-values-footer" onclick="wbte_reset_table_values();" style="width:150px;"><i class="fas fa-undo"></i> <?php esc_attr_e( 'Undo', 'woo-bulk-table-editor' ); ?></button>
					<button class="button" onclick="wbteSaveCustomTable();" style="width:150px;"><i class="fas fa-database"></i> <?php esc_attr_e( 'Save Changes', 'woo-bulk-table-editor' ); ?></button>
				</div>
			</div>
			<table class="wp-list-table widefat fixed striped posts" id="wbtetable_custom" style="width:100%;">
				<thead>
					<?php self::wbte_get_table_head( $functions ); ?>
				</thead>
				<tbody id="the-list"> 
					<?php  
					foreach ( $products as $product ) {
						$variations = ( is_a( $product, 'WC_Product' ) && ( $product->is_type( 'variable' ) ) ) ? $product->get_available_variations() : '';
						$variations = ( is_a( $product, 'WC_Product_Subscription' ) && ( $product->is_type( 'variable-subscription' ) ) ) ? $product->get_available_variations() : $variations; 

						if ( ! empty( $variations ) ) {
							foreach ( $variations as $variation ) {
								$variation_obj = wc_get_product( $variation['variation_id'] );
								self::wbte_get_table_row( $variation_obj, true, $functions );
							}
						} else {
							self::wbte_get_table_row( $product, false, $functions );
						}
					};
					?>
				</tbody>
				<tfoot><?php wp_nonce_field( 'wbte_id' ); ?></tfoot>
			</table>
			
			<div id="wbte-paging-bottom2" class="cas-div-paging" style="margin-top: 6px;">
				<span class="cas-pagination">
				<?php
				
				$args = array( // WPCS: XSS ok.
					'base'      => '%_%',
					'format'    => '?paged=%#%',
					'current'   => $page_num,
					'total'     => $total_pages,
					'prev_text' => is_rtl() ? '&rarr;' : '&larr;',
					'next_text' => is_rtl() ? '&larr;' : '&rarr;',
					'type'      => 'plain',
					'end_size'  => 1,
					'mid_size'  => 1,
				);
				echo wp_kses( paginate_links( $args ), 'post' );
				
				?>
				</span>
			</div>

			</div>
		<div>
		<!-- FINAL END DIV -->
		</div>		
		<?php
	}

	/**
	 * Get table head
	 */
	public static function wbte_get_table_head( $functions ) {

		$custom_fields = self::wbte_get_custom_fields_array();
		$sort_count    = 4;

		?>
		<tr>
			<th class="th-first" style="width:20px;"><input type="checkbox" id="checkall" onclick="checkVisible();"></th>
			<th>
				<a href="#" onclick="sort(1,'text');"><?php esc_html_e( 'Name', 'woo-bulk-table-editor' ); ?></a> <i id="s1" class="fas fa-sort" style="padding-right: 3px;padding-left:3px;"></i>
			</th>
			<th style="width:20px;"></th>
			<th style="width:100px;">
				<a href="#" onclick="sort(3,'text');"><?php esc_html_e( 'Type', 'woo-bulk-table-editor' ); ?></a> <i id="s3" class="fas fa-sort" style="padding-right: 3px;padding-left:3px;"></i></th>
			<th style="width:300px;"><?php esc_html_e( 'Attributes', 'woo-bulk-table-editor' ); ?></a></th>
			<?php foreach ( $custom_fields as $custom ) : ?>
				<?php $sort_count++; ?>
				<th><a href="#" onclick="sort(<?php echo esc_attr( $sort_count ); ?>, '<?php echo esc_attr( $custom->type ); ?>');"><?php echo esc_attr( $custom->name ); ?></a> <i id="s<?php echo esc_attr( $sort_count ); ?>" class="fas fa-sort" style="padding-right: 3px;padding-left:3px;"></i></th>
			<?php endforeach; ?>
		</tr>
		
		<tr class="th-info">
			<th></th>
			<th style="vertical-align: middle;">
				<strong><?php esc_html_e( 'Bulk Editor', 'woo-bulk-table-editor' ); ?></strong><br />
				<span class="th-desc"><?php esc_html_e( 'Enter values (input) then select bulk function.', 'woo-bulk-table-editor' ); ?>
				</span>
			</th>
			<th></th>
			<th></th>
			<th>
				<?php echo wp_kses( self::wbte_get_product_attributes_dropdown(), $functions->wbte_get_allowed_html() ); ?>
				<select class="input-select" id="wbte_attributes_action" onchange="wbte_bulk_add_attributes();" title="<?php esc_html_e( 'Bulk Add Attributes (Not for Variations)', 'woo-bulk-table-editor' ); ?>">
					<option value=""><?php esc_html_e( 'Select Action', 'woo-bulk-table-editor' ); ?></option>
					<option value="add"><?php esc_html_e( 'Add', 'woo-bulk-table-editor' ); ?></option>
					<option value="set"><?php esc_html_e( 'Set', 'woo-bulk-table-editor' ); ?></option>
					<option value="remove"><?php esc_html_e( 'Remove', 'woo-bulk-table-editor' ); ?></option>
					<option value="clear"><?php esc_html_e( 'Clear', 'woo-bulk-table-editor' ); ?></option>
				</select>
			</th>
			<?php foreach ( $custom_fields as $custom ) : ?>
				<th>
					<input type="text" name="<?php echo esc_attr( $custom->slug ); ?>_value" id="<?php echo esc_attr( $custom->slug ); ?>_value" class="input-txt"><br/>
					<select name="<?php echo esc_attr( $custom->slug ); ?>_select" id="<?php echo esc_attr( $custom->slug ); ?>_select" class="input-select" onchange="wbteBulkCustomValues('<?php echo esc_js( $custom->slug ); ?>');">
						<?php $functions->wbte_get_bulk_options( 'customs' ); ?>
					</select>
				</th>
			<?php endforeach; ?>
		</tr>
		<?php
	}

	/**
	 * Get table rows
	 */
	public static function wbte_get_table_row( $product, $is_variable, $functions ) {

		$product_name = $product->get_name();
		$attributes   = $product->get_attributes();
		$product_id   = $product->get_id();
		$type         = ( $is_variable ) ? 'var' : 'prod';
		$tag_ids      = $product->get_tag_ids();

		if ( $is_variable ) {
			$product_id = $product->get_parent_id();
			$tag_ids    = wc_get_product( $product_id )->get_tag_ids();

			if ( ! empty( $attributes ) && is_array( $attributes ) ) {
				foreach ( $attributes as $key => $val ) {
					if ( ! empty( $val ) ) {
						$product_name .= ', ' . ucfirst( $val );
					}
				}
			}
		}

		$tags          = $functions->wbte_get_product_tags( $tag_ids );
		$custom_fields = self::wbte_get_custom_fields_array();
	
		?>
		
		<tr class="lozad" style="vertical-align: top;" data-type="<?php echo esc_attr( $type ); ?>">
			<td id="<?php echo esc_attr( $product->get_id() ); ?>" class="cas-td-chk">
				<form id="frm_<?php echo esc_attr( $product->get_id() ); ?>_<?php echo esc_attr( wp_rand( 1, 50000 ) ); ?>">
				<input name="id" value="<?php echo esc_attr( $product->get_id() ); ?>" type="checkbox">
				<input type="hidden" name="saleprice" value="<?php echo esc_attr( $product->get_sale_price() ); ?>">
				<input type="hidden" name="tags" value="<?php echo esc_attr( $tags ); ?>">
				<input type="hidden" name="stock_status" value="<?php echo esc_attr( $product->get_stock_status() ); ?>">
				<input type="hidden" name="brands" value="<?php echo esc_attr( $functions->wbte_get_product_brands( $product_id, 'slug' ) ); ?>">
				<input type="hidden" name="parent_id" value="<?php echo esc_attr( $is_variable ? $product->get_parent_id() : '' ); ?>">
			</td>
			<td>
				<?php
				if ( ! $is_variable ) {
					?>
					<input type="text" class="input-txt-name" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="true">
					<?php
				} else {
					?>
					<input type="hidden" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="false">
					<?php echo esc_attr( $product_name ); ?>
					<?php
				}
				?>
			</td>
			<td>
				<a href="<?php echo esc_url( get_admin_url() . 'post.php?post=' . $product_id . '&action=edit' ); ?>" target="_new" title="<?php esc_attr_e( 'Edit product', 'woo-bulk-table-editor' ); ?>">
					<i class="fas fa-edit <?php echo esc_attr( ( $product->is_on_sale() ) ? 'cas-fa-sale' : '' ); ?>"></i>
				</a>
			</td>
			<td><?php echo esc_attr( ucfirst( $product->get_type() ) ); ?></td>
			<td>
				<?php self::wbte_get_product_attributes( $product ); ?>
			</td>
			<?php foreach ( $custom_fields as $custom ) : ?>
				<td>
					<input type="text" class="input-txt-name" data-iscustom="1" name="<?php echo esc_attr( $custom->slug ); ?>" value="<?php echo esc_attr( get_post_meta( $product->get_id(), $custom->slug, true ) ); ?>">
				</td>
			<?php endforeach; ?>
			</form>
		</tr>
		<?php
	}

	/**
	 * Get product attributes
	 */
	public static function wbte_get_product_attributes( $product ) {

		
		if ( empty( $product ) ) {
			return '';
		}

		$attributes      = $product->get_attributes();
		$attribute_list  = '';
		$attr_input_list = '';
		$is_variation    = $product->is_type('variation');
		$variation_css   = 'background:white;color:black;';

		if ( $is_variation ) {

			$variation_attributes = $product->get_variation_attributes();
			$variation_css        = 'background:whitesmoke;color:gray;';
			$output               = '';

			foreach ( $variation_attributes as $attribute_slug => $attribute_value ) {
				
				$taxonomy = str_replace( 'attribute_', '', $attribute_slug );

				// Check if the attribute is a taxonomy-based attribute
				if ( taxonomy_exists( $taxonomy ) ) {
					$term = get_term_by( 'slug', $attribute_value, $taxonomy );
					if ($term) {
						$output .= wc_attribute_label( $taxonomy ) . ': ' . $term->name . '<br/>';
					}
				} else {
					// Custom attribute
					$output .= wc_attribute_label( $taxonomy ) . ': ' . $attribute_value . '<br/> ';
				}
			}
			$attribute_list = $output;
		}


		if ( ! empty( $attributes ) && ! $is_variation ) {
			foreach ( $attributes as $attribute ) {
				
				if ( $attribute->is_taxonomy() ) {
					// Taxonomy-based attribute
					$terms  = wp_get_post_terms( $product->get_id(), $attribute->get_name(), 'all');
					$values = array();
					foreach ( $terms as $term ) {
						$values[] = esc_html( $term->name );
					}
					$attribute_list  .= wc_attribute_label( $attribute->get_name() ) . ': ' . implode( ', ', $values ) . '<br>';
					$attr_input_list .= wc_attribute_label( $attribute->get_name() ) . '=' . implode( ',', $values ) . ';';
				} else {
					// Custom attribute
					$values           = $attribute->get_options();
					$attribute_list  .= wc_attribute_label( $attribute->get_name() ) . ': ' . implode( ', ', $values ) . '<br>';
					$attr_input_list .= wc_attribute_label( $attribute->get_name() ) . '=' . implode( ',', $values ) . ';';
				}
				
			}
		}

		?>
		<input type="hidden" name="attributes" value="<?php echo esc_attr( strtolower( $attr_input_list ) ); ?>" data-is-variation="<?php echo esc_attr( $is_variation ); ?>">
		<div id="<?php echo esc_attr( $product->get_id() ); ?>_attributes_txt" style="max-height:60px;overflow:scroll;padding-left:4px;<?php echo esc_attr( $variation_css ); ?>" title="<?php esc_attr_e( 'Scroll enabled if many', 'woo-bulk-table-editor' ); ?>">
		<?php 
		echo wp_kses( $attribute_list, 'post' );
		?>
		</div>
		<?php

	}

	/**
	 * Get all unique product attributes using a custom SQL query with transients
	 */
	public static function wbte_get_all_product_attributes() {
		
		$transient_key        = 'wbte_all_product_attributes';
		$transient_expiration = 10 * MINUTE_IN_SECONDS;
		$product_attributes   = get_transient( $transient_key ); //set to false if testing

		if ( false === $product_attributes ) {
			
			$product_attributes = array();

			// Get all registered attribute taxonomies
			$all_taxonomies = wc_get_attribute_taxonomy_names();

			// Arguments for WP_Query
			$args = array(
				'post_type'      => 'product',
				'post_status'    => 'publish',
				'posts_per_page' => -1,
				'meta_query'     => array(
					array(
						'key'     => '_product_attributes',
						'compare' => 'EXISTS',
					),
				),
			);

			// Execute the query
			$query = new WP_Query( $args );

			// Get the product IDs
			$product_ids = wp_list_pluck( $query->posts, 'ID' );
			
			if ( ! empty( $product_ids ) ) {
				$products = wc_get_products( array( 'include' => $product_ids ) );
				
				foreach ( $products as $product ) {
					$attributes = $product->get_attributes();
					foreach ( $attributes as $attribute ) {
						$name  = $attribute->get_name();
						$label = wc_attribute_label( $name );
						
						if ( ! isset( $product_attributes[$label] ) ) {
							$product_attributes[$label] = array();
						}
				
						if ( $attribute->is_taxonomy() ) {
							// Taxonomy-based attribute
							$terms = get_terms( array(
								'taxonomy'   => $name,
								'hide_empty' => false,
								'fields'     => 'names'
							));
							
							if ( ! is_wp_error( $terms ) && is_array( $terms ) ) {
								$product_attributes[$label] = array_merge( $product_attributes[$label], $terms );
							}

						} else {
							// Custom attribute
							$options = $attribute->get_options();
							if ( is_array( $options ) ) {
								$product_attributes[$label] = array_merge( $product_attributes[$label], $options );
							}
						}
					}
				}

				// Add missing taxonomies with empty arrays to the product attributes array
				foreach ( $all_taxonomies as $taxonomy ) {
					
					// Add missing taxonomies with their terms
					$label = wc_attribute_label( $taxonomy );
					
					if ( ! isset( $product_attributes[$label] ) ) {
						$product_attributes[$label] = array();
					}
					
					// Ensure all terms for this taxonomy are included
					$terms = get_terms( array(
						'taxonomy'   => $taxonomy,
						'hide_empty' => false,
						'fields'     => 'names'
					));

					if ( ! is_wp_error( $terms ) && is_array( $terms ) ) {
						$product_attributes[$label] = array_merge( $product_attributes[$label], $terms );
					}
				}

				// Remove duplicate terms and sort them
				foreach ( $product_attributes as $label => $terms ) {
					$product_attributes[$label] = array_unique( $terms );
					sort( $product_attributes[$label] );
				}
			}

			// Set the transient with the fetched data
			set_transient( $transient_key, $product_attributes, $transient_expiration );
		}

		return $product_attributes;
	}


	/**
	 * Generate dropdown HTML for product attributes
	 */
	public static function wbte_get_product_attributes_dropdown() {
		
		$attributes = self::wbte_get_all_product_attributes();

		if ( empty( $attributes ) ) {
			return esc_html_e( 'No Attributes Found', 'woo-bulk-table-editor' );
		}

		$dropdown = '<select id="bulk_add_attribute" name="bulk_add_attribute" class="input-select">';
		$dropdown .= '<option value="">' . __( 'Select Attributes', 'woo-bulk-table-editor' ) . '</option>';
		foreach ( $attributes as $label => $terms ) {
			$dropdown .= '<optgroup label="' . esc_attr( $label ) . '">';
			foreach ( $terms as $term ) {
				$dropdown .= '<option value="' . esc_attr( strtolower( $label ) . '=' ) . esc_attr( $term ) . '">' . esc_html( $term ) . '</option>';
			}
			$dropdown .= '</optgroup>';
		}
		$dropdown .= '</select>';

		return $dropdown;
	}

}

