<?php
/**
 * Bulk Table Editor functions
 *
 * @package BulkTableEditor/includes
 */

defined( 'ABSPATH' ) || exit;

require_once __DIR__ . '/class-wbte-templates.php';

/**
 * Class for functions
 */
class WbteFunctions {

	/**
	 * Update product
	 *
	 * @param var $post object.
	 */
	public function wbte_update_product( $post ) {

		if ( empty( $post['id'] ) ) {
			wp_die();
		}

		$custom_price_info = $this->wbte_get_custom_price_info();
		$options           = get_option( 'wbte_options' );
		$show_sku          = ( ! empty( $options[ 'wbte_use_sku_main_page' ] ) ) ? $options[ 'wbte_use_sku_main_page' ] : 'no';
		$show_vendors      = ( ! empty( $options[ 'wbte_vendor_integration' ] ) ) ? $options[ 'wbte_vendor_integration' ] : 'no';
		$disable_desc      = ( ! empty( $options[ 'wbte_disable_description' ] ) ) ? $options[ 'wbte_disable_description' ] : 'no';
		$id                = sanitize_text_field( $post['id'] );
		$price             = sanitize_text_field( $post['price'] );
		$sale_price        = sanitize_text_field( $post['saleprice'] );
		$stock             = sanitize_text_field( $post['stock'] );
		$sale_time_from    = ( ! empty( $post['salefrom_time'] ) ) ? $post['salefrom_time'] : '00:00';
		$sale_time_to      = ( ! empty( $post['saleto_time'] ) ) ? $post['saleto_time'] : '23:59';
		$sale_from         = ( ! empty( $post['salefrom'] ) ) ? $this->wbte_date_to_time( $post['salefrom'] . ' ' . $sale_time_from ) : '';
		$sale_to           = ( ! empty( $post['saleto'] ) ) ? $this->wbte_date_to_time( $post['saleto'] . ' ' . $sale_time_to ) : '';
		$name              = sanitize_text_field( $post['name'] );
		$update_name       = sanitize_text_field( $post['name_update'] );
		$description       = isset( $post['description'] ) ? wc_clean( $post['description'] ) : '';
		$custom_price      = '';
		$product           = wc_get_product( intval( $id ) );
		$prev_sale_price   = '_wbte_prev_sale_price';
		$meta_time_from    = '_wbte_sale_time_from';
		$meta_time_to      = '_wbte_sale_time_to';
		$meta_subscription = '_subscription_price';
		$prev_price        = '_wbte_prev_price';
		$brands            = ( 'yes' === $options['wbte_use_brands'] ) ? sanitize_text_field( $post['brands'] ) : '';

		//Update custom price
		if ( 'yes' === $custom_price_info['active'] && 'no' === $show_sku ) {

			$custom_price = sanitize_text_field( $post['customprice'] );
			$product->update_meta_data( $custom_price_info['price'], $custom_price );

		} elseif ( 'yes' === $show_sku ) {

			$sku = sanitize_text_field( $post['sku'] );
			if ( strlen( $sku ) > 0 ) {
				$is_unique = wc_product_has_unique_sku( $id, $sku );
				if ( $is_unique ) {
					$product->set_sku( $sku );
				}
			} else {
				$product->set_sku( '' );
			}

		}

		//Update vendor
		if ( 'yes' === $show_vendors ) {
			$vendor_id = sanitize_text_field( $post['vendor'] );
			$_id       = $product->get_id();

			if ( $product->get_parent_id() > 0 ) {
				$_id = $product->get_parent_id();
			}

			wp_set_object_terms( $_id, absint( $vendor_id ), WC_PRODUCT_VENDORS_TAXONOMY );
		}

		//Update brands
		if ( ! empty( $brands ) ) {
			$brands_set = explode( ',', $brands );
			wp_set_object_terms( ! empty( $product->get_parent_id() ) ? $product->get_parent_id() : $product->get_id(), $brands_set, 'product_brand' );
		}

		if ( 'subscription' === $product->get_type() ) {
			$product->update_meta_data( $meta_subscription, $price );
		} else {
			$product->set_price( $price );
		}

		$product->update_meta_data( $prev_price, $product->get_regular_price() );
		$product->set_regular_price( $price );
		
		//Product name
		if ( 'true' === $update_name && strlen( $name ) > 0 ) {
			$product->set_name( $name );
		}

		//Description
		if ( 'yes' !== $disable_desc ) {
			$description = ( $description ) ? wp_filter_post_kses( $post['description'] ) : '';
			$product->set_description( $description );
		}

		// Stock
		$stock_status = ( ! empty( $product->get_stock_status() ) ) ? $product->get_stock_status() : 'instock';
		$manage_stock = isset( $stock ) && '' !== $stock; // Check if stock is set and not empty

		$product->set_manage_stock( $manage_stock );

		if ( $manage_stock ) {
			if ( 0 === $stock ) {
				// If stock is exactly 0
				$product->set_stock_quantity( 0 );
				$stock_status = 'outofstock';
			} else {
				// If stock is greater than 0
				$product->set_stock_quantity( $stock );
				$stock_status = $stock > 0 ? 'instock' : 'outofstock';
			}
		} else if ( 'yes' === $options['wbte_no_stock_management'] ) {
			// Automatic stock management is turned on
			$product->set_manage_stock( false );
			$product->set_stock_quantity( null ); // Nullify stock quantity when not managing stock
			$stock_status = 'instock';
		}

		// Set stock status
		$product->set_stock_status( $stock_status );
		
		//Sale
		if ( ! empty( $sale_price ) ) { 
			
			//Normal
			$product->set_sale_price( $sale_price );
			$product->update_meta_data( $prev_sale_price, $sale_price );
			$product->update_meta_data( $meta_time_from, $sale_time_from );
			$product->update_meta_data( $meta_time_to, $sale_time_to );

			$product->set_date_on_sale_from( $sale_from );
			$product->set_date_on_sale_to( $sale_to );

		} elseif ( '' === $sale_price && '' === $sale_from && '' === $sale_to ) {
			// Clear sales
			$product->set_sale_price( '' );
			$product->set_date_on_sale_from( '' );
			$product->set_date_on_sale_to( '' );
			$product->update_meta_data( $meta_time_from, '' );
			$product->update_meta_data( $meta_time_to, '' );
		} 

		$product->save();
		
	}
	

	/**
	 * Convert a date string into a timestamp
	 *
	 * @param var @date_string date.
	 */
	public function wbte_date_to_time( $date_string ) {

		if ( 0 === $date_string ) {
			return 0;
		}

		$date_time = new WC_DateTime( $date_string, new DateTimeZone( wp_timezone_string() ) );

		$date_time->setTimezone( new DateTimeZone('UTC') );

		return intval( $date_time->getTimestamp() );

	}

	/**
	 * Update product ext
	 *
	 * @param var $post object.
	 */
	public function wbte_update_product_ext( $post ) {

		if ( ! isset( $post['id'] ) ) {
			wp_die();
		}

		$options      = get_option( 'wbte_options' );
		$id           = sanitize_text_field( $post['id'] );
		$featured     = sanitize_text_field( $post['featured'] );
		$sku          = sanitize_text_field( $post['sku'] );
		$backorder    = sanitize_text_field( $post['backorder'] );
		$stock_status = sanitize_text_field( $post['instock'] );
		$weight       = sanitize_text_field( $post['weight'] );
		$length       = sanitize_text_field( $post['length'] );
		$width        = sanitize_text_field( $post['width'] );
		$height       = sanitize_text_field( $post['height'] );
		$terms        = sanitize_text_field( $post['tags'] );
		$name         = sanitize_text_field( $post['name'] );
		$update_name  = sanitize_text_field( $post['name_update'] );
		$image        = sanitize_text_field( $post['product_img'] );
		$parent_id    = sanitize_text_field( $post['parent_id'] );
		$visibility   = sanitize_text_field( $post['visibility'] );
		$shipping     = sanitize_text_field( $post['shipping'] );
		$description  = isset( $post['description'] ) ? wc_clean( $post['description'] ) : '';
		$product      = wc_get_product( $id );
		$disable_desc = ( strlen( $options[ 'wbte_disable_description' ] ) > 0 ) ? $options[ 'wbte_disable_description' ] : 'no';

		//Set values
		$product->set_featured( $featured );
	
		if ( ! empty( $sku ) ) {
			$is_unique = wc_product_has_unique_sku( $id, $sku );
			if ( $is_unique ) {
				$product->set_sku( $sku );
			}
		} else {
			$product->set_sku( '' );
		}

		//Stock
		if ( ! empty( $product->get_stock_quantity() ) ) {
			$product->set_manage_stock( true );
		} else {
			if ( 'yes' === $options['wbte_no_stock_management'] ) {
				$product->set_manage_stock( false );
			} else {
				$product->set_manage_stock( true );
			}
		}

		//Set stock status
		$product->set_stock_status( $stock_status );

		$product->set_backorders( $backorder );
		$product->set_catalog_visibility( $visibility );
		$product->set_weight( $weight );
		$product->set_length( $length );
		$product->set_width( $width );
		$product->set_height( $height );
		$product->set_image_id( $image );
		$product->set_shipping_class_id( $shipping );

		//Set terms
		$term_arr  = array();
		$term_list = explode( ',', $terms );
		
		foreach ( $term_list as $obj ) {
			array_push( $term_arr, rtrim( $obj, ' ' ) );
		}

		wp_set_object_terms( $id, $term_arr, 'product_tag', false );
		
		//If variable - set tags on parent product
		if ( ! empty( $parent_id ) ) {
			wp_set_object_terms( $parent_id, $term_arr, 'product_tag', false );
		}
		
		//Product name
		if ( 'true' === $update_name && ! empty( $name ) ) {
			$product->set_name( $name );
		}

		//Description
		if ( 'yes' !== $disable_desc ) {
			$description = ( $description ) ? wp_filter_post_kses( $post['description'] ) : '';
			$product->set_description( $description );
		}

		$product->save();

	}

	/**
	 * Update Categories and Images
	 */
	public function wbte_update_categories_images( $post ) {
	
		if ( ! isset( $post['id'] ) ) {
			wp_die();
		}

		$id              = sanitize_text_field( $post['id'] );
		$name            = sanitize_text_field( $post['product_name'] );
		$name_update     = sanitize_text_field( $post['name_update'] );
		$product_img     = sanitize_text_field( $post['product_img'] );
		$product_img_ids = sanitize_text_field( $post['product_img_ids'] );
		$parent_id       = sanitize_text_field( $post['parent_id'] );
		$categories      = sanitize_text_field( $post['categories'] );
		$product         = wc_get_product( $id );
		$parent          = '';
		$product_img_ids = array_map( 'intval', explode( ',', $product_img_ids ) );
		$categories      = array_map( 'intval', explode( ',', $categories ) );

		if ( ! empty( $product->get_parent_id() ) ) {
			$parent = wc_get_product( $product->get_parent_id() );
		}

		//Product name
		if ( 'true' === $name_update && ! empty( $name ) ) {
			$product->set_name( $name );
		}

		//Categories
		if ( ! empty( $categories ) ) {
			if ( ! empty( $parent ) ) {
				//Is variable
				$parent->set_category_ids( $categories );
			} else {
				//Simple
				$product->set_category_ids( $categories );
			}
		}

		//Featured
		$product->set_image_id( $product_img );
		
		//Images
		if ( ! empty( $product_img_ids ) ) {
			if ( ! empty( $parent ) ) {
				//Is variable
				$parent->set_gallery_image_ids( $product_img_ids );
			} else {
				//Simple
				$product->set_gallery_image_ids( $product_img_ids );
			}
		}

		//Save changes on parent - if variable product
		if ( ! empty( $parent ) ) {
			$parent->save();
		}

		$product->save();

	}

	/**
	 * Save attributes and custom fields
	 */
	public function wbte_save_attributes_and_custom( $post ) {

		if ( ! isset( $post['id'] ) ) {
			wp_die();
		}
	
		$id          = sanitize_text_field( $post['id'] );
		$name        = sanitize_text_field( $post['name'] );
		$name_update = sanitize_text_field( $post['name_update'] );
		$parent_id   = sanitize_text_field( $post['parent_id'] );
		$product     = wc_get_product( $id );
		$parent      = '';
	
		if ( ! empty( $parent_id ) ) {
			$parent = wc_get_product( $product->get_parent_id() );
		}
	
		// Update product name if needed
		if ( 'true' === $name_update && ! empty( $name ) ) {
			$product->set_name( $name );
		}

		// Process and save attributes (for non-variable products)
		if ( empty( $parent ) && isset( $post['attributes'] ) ) {
			
			$attributes = sanitize_text_field( $post['attributes'] );
	
			// If attributes are empty, remove all attributes
			if ( empty( $attributes ) ) {
				$product->set_attributes( array() );
			} else {
				$attributes_array = explode( ';', $attributes );
				$attributes_data  = array();
	
				foreach ( $attributes_array as $attribute ) {
					list( $key, $value ) = explode( '=', $attribute );
					if ( ! empty( $key ) ) {
						$attributes_data[ 'pa_' . $key ] = $value;
					}
				}
	
				$product_attributes = $product->get_attributes();
	
				// Remove attributes that are not in the new attributes data
				foreach ( $product_attributes as $key => $attribute ) {
					if ( ! array_key_exists( $key, $attributes_data ) ) {
						unset( $product_attributes[ $key ] );
					}
				}
	
				// Update existing or add new attributes
				foreach ( $attributes_data as $key => $value ) {
					if ( isset( $product_attributes[ $key ] ) ) {
						$product_attributes[ $key ]->set_options( array( $value ) );
					} else {
						$product_attributes[ $key ] = new WC_Product_Attribute();
						$product_attributes[ $key ]->set_id( wc_attribute_taxonomy_id_by_name( $key ) );
						$product_attributes[ $key ]->set_name( $key );
						$product_attributes[ $key ]->set_options( array( $value ) );
						$product_attributes[ $key ]->set_position( 0 );
						$product_attributes[ $key ]->set_visible( true );
						$product_attributes[ $key ]->set_variation( false );
					}
				}
	
				$product->set_attributes( $product_attributes );
			}
		}
	
		// Process and save custom fields
		if ( isset( $post['custom_fields'] ) && ! empty( $post['custom_fields'] ) ) {
			foreach ( $post['custom_fields'] as $key => $value ) {
				update_post_meta( $id, sanitize_text_field( $key ), sanitize_text_field( $value ) );
			}
		}
	
		// Save the product
		$product->save();
	}

	/**
	 * Move product to trash
	 *
	 * @param var $post product.
	 */
	public function wbte_move_to_trash( $post ) {
		
		if ( ! isset( $post['id'] ) ) {
			die();
		}

		$id      = sanitize_text_field( $post['id'] );
		$product = wc_get_product( $id );

		$product->set_status( 'trash' );
		$product->save();

	}

	/**
	 * Get category slug
	 *
	 * @param var $id int.
	 */
	public function wbte_get_category_slug( $id ) {
		$term = get_term_by( 'id', $id, 'product_cat', 'ARRAY_A' );
		$term = ( isset( $term['slug'] ) ) ? $term['slug'] : null;
		return $term;
	}

	/**
	 * Get products
	 */
	public function wbte_get_woo_products() {

		$wbte_options  = get_option( 'wbte_options' );
		$post_per_page = ( $wbte_options['wbte_posts_per_page'] > 0 ) ? $wbte_options['wbte_posts_per_page'] : 50;
		$is_download   = ( ! empty( filter_input( 1, 'csv', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'csv', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$prod_search   = filter_input( 1, 'product_search', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$prod_order    = filter_input( 1, 'order', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$page_num      = ( ! empty( filter_input( 1, 'paged', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'paged', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : 1;
		$stype         = filter_input( 1, 'stype', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$post_orderby  = ( isset( $wbte_options['wbte_product_orderby'] ) && strlen( $wbte_options['wbte_product_orderby'] ) > 0 ) ? $wbte_options['wbte_product_orderby'] : 'name';
		$post_order    = ( isset( $wbte_options['wbte_product_order'] ) && strlen( $wbte_options['wbte_product_order'] ) > 0 ) ? $wbte_options['wbte_product_order'] : 'ASC';
		$del_transient = false;
		$cat_in_search = ( ! empty( $wbte_options['wbte_inlcude_cat_in_search'] ) && 'yes' === $wbte_options['wbte_inlcude_cat_in_search'] ) ? true : false;

		$default_category = ! empty( $wbte_options['wbte_product_cat'] ) ? get_term_by( 'id', $wbte_options['wbte_product_cat'], 'product_cat' )->slug : '';
		$form_submitted   = filter_input( INPUT_GET, 'form_submitted', FILTER_VALIDATE_INT );

		if ( $form_submitted ) {
			// Get the categories from the form submission
			$categories = filter_input( INPUT_GET, 'categories', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		} else {
			// Use the default category if the form was not submitted
			$categories = $default_category;
		}

		if ( ! empty( $prod_search ) ) {
			$del_transient = true;
			if ( ! $cat_in_search ) {
				$categories    = '';
			}
		}

		$args = array(
			'orderby' => $post_orderby,
			'order'   => $post_order,
		);
		
		$query = new WC_Product_Query( $args );

		//Product statuses
		if ( is_array( $wbte_options['wbte_product_status'] ) && count( $wbte_options['wbte_product_status'] ) > 0 ) {
			$query->set( 'status', $wbte_options['wbte_product_status'] );
		}

		//Categories
		if ( ! empty( $categories ) ) {
			$query->set( 'tax_query', array(
				array(
					'taxonomy' => 'product_cat',
					'field'    => 'slug',
					'terms'    => explode( ',', $categories ),
					'operator' => 'IN',
				),
			) );
		}

		if ( ! empty( $prod_search ) ) {
			if ( 'sku' === $stype ) {
				$query->set( 'sku', $prod_search );
			} else {
				$query->set( 's', $prod_search );
			}
			$query->set( 'orderby', 'relevance' );
			$del_transient = true;
		}

		if ( $prod_order ) {
			$query->set( 'order', $prod_order );
		}
		
		if ( $post_per_page ) {
			
			$query->set( 'page', $page_num );
			$query->set( 'paginate', true );
			$query->set( 'limit', $post_per_page );
			
			if ( get_transient( 'cas_json_products' ) && get_transient( 'cas_json_products' )['page'] !== $page_num ) {
				$del_transient = true;
			}
		}

		//If download
		if ( 'true' === $is_download ) {

			$query->set( 'limit', -1 );
			$del_transient = true;

		}

		if ( $del_transient ) {
			delete_transient( 'cas_json_products' );
		}

		$query = apply_filters( 'wbte_get_woo_products', $query );

		return $query;
	}
	

	/**
	 * Get product categories for settings - select box
	 *
	 * @param var $wbte_options object.
	 */
	public function wbte_settings_get_categories_select( $wbte_options ) {

		$args = array(
			'hide_empty'        => 1,
			'taxonomy'          => 'product_cat',
			'hierarchical'      => 1,
			'show_count'        => 1,
			'name'              => 'wbte_options[wbte_product_cat]',
			'orderby'           => 'menu_order',
			'order'             => 'asc',
			'show_option_none'  => __( 'All products', 'woo-bulk-table-editor' ),
			'option_none_value' => '',
			'selected'          => $wbte_options['wbte_product_cat'],
		);
		
		wp_dropdown_categories( $args );
	}

	/**
	 * Create dropdown categories (for multiple select)
	 */
	public static function wbte_create_category_dropdown( $id, $name = '' ) {

		$options    = get_option( 'wbte_options' );
		$hide_empty = ! empty( $options['wbte_categories_all'] ) && 'yes' ===  $options['wbte_categories_all'] ? 0 : 1;
		$order_by   = ! empty( $options['wbte_categories_orderby'] ) ? $options['wbte_categories_orderby'] : 'menu_order';
		
		$args = array(
			'taxonomy'          => 'product_cat',
			'hide_empty'        => $hide_empty,
			'hierarchical'      => 1,
			'pad_counts'        => 1,
			'show_count'        => 1,
			'value_field'       => 'slug',
			'orderby'           => $order_by,
			'option_none_value' => '',
			'show_option_none'  => __( 'Select Categories ( All )', 'woo-bulk-table-editor' ),
			'id'                => $id,
			'name'              => $name,
		);

		wp_dropdown_categories( $args );
	}

	/**
	 * Get product categories - select box
	 *
	 * @param var $prodcat object.
	 */
	public function wbte_get_categories_select( $prodcat ) {
		
		$options    = get_option( 'wbte_options' );
		$view       = ( ! empty( filter_input( 1, 'view', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'view', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : 'prod';
		$filter     = ( ! empty( filter_input( 1, 'sales_filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'sales_filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$hide_empty = ! empty( $options['wbte_categories_all'] ) && 'yes' ===  $options['wbte_categories_all'] ? 0 : 1;
		$order_by   = ! empty( $options['wbte_categories_orderby'] ) ? $options['wbte_categories_orderby'] : 'menu_order';

		$args = array(
			'pad_counts'         => 1,
			'show_count'         => 1,
			'hierarchical'       => 1,
			'hide_empty'         => $hide_empty,
			'show_uncategorized' => 1,
			'orderby'            => $order_by,
			'selected'           => $prodcat,
			'show_option_none'   => __( 'Select Category ( All )', 'woo-bulk-table-editor' ),
			'option_none_value'  => '',
			'value_field'        => 'slug',
			'taxonomy'           => 'product_cat',
			'name'               => 'product_cat',
			'class'              => 'dropdown_product_cat',
		);
		?>
		
		<form name="gc" id="wbte-cat-select" type="get" action="edit.php" onchange="submit();">  
		<input type="hidden" name="post_type" value="product">  
		<input type="hidden" name="page" value="wbte-products">
		<input type="hidden" name="view" value="<?php echo esc_attr( $view ); ?>">
		<input type="hidden" name="sales_filter" value="<?php echo esc_attr( $filter ); ?>">
		<?php
			wp_dropdown_categories( apply_filters( 'wbte_category_selector_args', $args ) );
		?>
		</form>
		<?php
	}

	/**
	 * Get filter for sales products and variations
	 */
	public function wbte_get_sales_products_select() {

		$wbte_options = get_option( 'wbte_options' );
		$view         = ( ! empty( filter_input( 1, 'view', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'view', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : 'prod';
		$filter       = ( ! empty( filter_input( 1, 'sales_filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'sales_filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$selected     = 'selected="selected"';
		$default      = '';
		$is_true      = '';
		$is_false     = '';

		switch ( $filter ) {
			case '':
				$default = $selected;
				break;
			case 'onSale':
				$is_true = $selected;
				break;
			case 'noSale':
				$is_false = $selected;
				break;
		}

		?>
		<form name="wbte-filter-sales" id="wbte-filter-sales" onchange="wbteFilterSales();">
			<input type="hidden" name="post_type" value="product">  
			<input type="hidden" name="page" value="wbte-products">
			<input type="hidden" name="view" value="<?php echo esc_attr( $view ); ?>">  
			<select name="sales_filter" id="sales_filter" multiselect="multiselect" style="margin-top:0px !important;">
				<option value="" <?php echo esc_attr( $default ); ?>><?php esc_attr_e( 'Sale Status ( All )', 'woo-bulk-table-editor' ); ?></option>
				<option value="onSale" <?php echo esc_attr( $is_true ); ?>><?php esc_attr_e( 'On Sale', 'woo-bulk-table-editor' ); ?></option>
				<option value="noSale" <?php echo esc_attr( $is_false ); ?>><?php esc_attr_e( 'Not On Sale', 'woo-bulk-table-editor' ); ?></option>
			</select>
		</form>
		<?php
	}

	/**
	 * Get filter for tags
	 */
	public function wbte_get_tags_filter() {
		
		$filter = ( ! empty( filter_input( 1, 'tags-filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'tags-filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		
		$args = array(
			'show_option_none'   => __( 'Tags ( All )', 'woo-bulk-table-editor' ),
			'selected'           => $filter,
			'option_none_value'  => '',
			'hide_empty'         => 0,
			'orderby'            => 'name',
			'value_field'        => 'name',
			'taxonomy'           => 'product_tag',
			'name'               => 'product_tag',
			'class'              => 'dropdown_product_tag',
			'show_count'         => 0,
		);
		?>
		<form name="tags-filter" id="tags-filter" onchange="wbteFilterSales();">  
		<?php
			wp_dropdown_categories( apply_filters( 'wbte_tags_selector_args', $args ) );
		?>
		</form>
		<?php
	}

	/**
	 * Get filter for brands (if present)
	 */
	public function wbte_get_brands_filter() {
		
		$filter = ( ! empty( filter_input( 1, 'brands-filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'brands-filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		
		$args = array(
			'show_option_none'   => __( 'Brands ( All )', 'woo-bulk-table-editor' ),
			'selected'           => $filter,
			'hierarchical'       => 1,
			'option_none_value'  => '',
			'hide_empty'         => 0,
			'orderby'            => 'name',
			'value_field'        => 'slug',
			'taxonomy'           => 'product_brand',
			'name'               => 'product_brand',
			'class'              => 'dropdown_product_brands',
			'show_count'         => 0,
		);
		?>
		<form name="brands-filter" id="brands-filter" onchange="wbteFilterSales();">  
		<?php
			wp_dropdown_categories( apply_filters( 'wbte_brands_selector_args', $args ) );
		?>
		</form>
		<?php
	}

	/**
	 * Get brands as a select dropdown
	 */
	public function wbte_get_brands_select() {
		
		$args = array(
			'show_option_none'   => __( 'Select Brands', 'woo-bulk-table-editor' ),
			'selected'           => '',
			'hierarchical'       => 1,
			'option_none_value'  => '',
			'hide_empty'         => 0,
			'orderby'            => 'name',
			'value_field'        => 'slug',
			'taxonomy'           => 'product_brand',
			'name'               => 'product_brand_select',
			'show_count'         => 0,
		);
		if ( class_exists( 'WC_Brands' ) ) {
			wp_dropdown_categories( $args );
		}
	}

	/**
	 * Get Stock Status filter
	 */
	public function wbte_get_stock_status_filter() {
		
		$option_blank       = __( 'Stock Status (All)', 'woo-bulk-table-editor' );
		$option_instock     = __( 'In stock', 'woo-bulk-table-editor' );
		$option_outofstock  = __( 'Out of stock', 'woo-bulk-table-editor' );
		$option_onbackorder = __( 'On backorder', 'woo-bulk-table-editor' );
		$filter             = ( ! empty( filter_input( 1, 'stock_status', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'stock_status', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
	
		?>
		<div>
			<form name="stock-filter" id="stock-filter" onchange="wbteFilterSales();">  
			<select name="stock_status" id="stock_status">
			<option value="" <?php echo esc_attr( '' === $filter ? 'selected="selected"' : '' ); ?>><?php echo esc_html( $option_blank ); ?></option>
			<option value="instock" <?php echo esc_attr( 'instock' === $filter ? 'selected="selected"' : '' ); ?>><?php echo esc_html( $option_instock ); ?></option>
			<option value="outofstock" <?php echo esc_attr( 'outofstock' === $filter ? 'selected="selected"' : '' ); ?>><?php echo esc_html( $option_outofstock ); ?></option>
			<option value="onbackorder" <?php echo esc_attr( 'onbackorder' === $filter ? 'selected="selected"' : '' ); ?>><?php echo esc_html( $option_onbackorder ); ?></option>
			</select>
			</form>
		</div>
		<?php
	}

	/**
	 * Get product tags
	 */
	public function wbte_get_product_tags( $tag_ids ) {

		$tags = '';

		if ( ! empty( $tag_ids ) ) {
			foreach ( $tag_ids as $obj ) {
				$tags .= get_term( $obj )->name . ', ';
			}
			$tags = substr( $tags, 0, strlen( $tags ) - 2 );
		}

		return $tags;
	}

	/**
	 * Get custom price info - array()
	 */
	public function wbte_get_custom_price_info() {

		$options   = get_option( 'wbte_options' );
		$price     = $options[ 'wbte_custom_price_1' ];
		$name      = $options[ 'wbte_custom_price_1_header' ];
		$is_active = $options[ 'wbte_custom_price_1_visible' ];
		$calc_type = $options[ 'wbte_custom_price_1_normal_calc' ];
		
		$price_arr = array(
			'price'       => $price,
			'name'        => $name,
			'active'      => $is_active,
			'normal_calc' => $calc_type,
		);
		
		return apply_filters( 'wbte_custom_price_array', $price_arr );
	}

	/**
	 * Finds and returns SKU
	 */
	public function wbte_get_product_sku( $product ) {
		
		$sku       = $product->get_sku();
		$is_unique = wc_product_has_unique_sku( $product->get_id(), $sku );
		
		if ( ! $is_unique ) {
			$sku = '';
		}

		return $sku;
	}

	/**
	 * Get sale in percent
	 *
	 * @param var $product product object.
	 */
	public function wbte_get_sale_percent( $product ) {

		$sale_percent = '';

		if ( $product->is_on_sale() && floatval( $product->get_regular_price() ) > 0 ) {
			$price_reg = $product->get_regular_price() ? floatval( $product->get_regular_price() ) : 0;
			$saleprice = $product->get_sale_price() ? floatval( $product->get_sale_price() ) : 0;
			$reduction = round( ( $price_reg - $saleprice ) * 100 / $price_reg );
			if ( ! $reduction ) {
				$reduction = '';
			}
			$sale_percent = $reduction;
		}

		return $sale_percent;
	}

	/**
	 * Get JSON products
	 *
	 */
	public function wbte_get_json_products() {

		$query          = $this->wbte_get_woo_products();
		$trans_products = get_transient( 'cas_json_products' );
		$stype          = filter_input( 1, 'stype', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$is_download    = ( ! empty( filter_input( 1, 'csv', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'csv', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
	
		if ( $trans_products && '' === $stype && empty( $is_download ) ) {
			return $trans_products;
		}

		$product_obj = $query->get_products();
		$page_num    = ( ! empty( filter_input( 1, 'paged', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'paged', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : 1;
		
		$json_products = array( 
			'rows'      => array(), 
			'total'     => 0,
			'num-pages' => 0,
			'page'      => $page_num,
		);

		if ( $product_obj ) {
			
			$json_products['rows'][]    = $product_obj->products;
			$json_products['total']     = $product_obj->total;
			$json_products['num-pages'] = $product_obj->max_num_pages;

			//Set transient
			set_transient( 'cas_json_products', $json_products, 3 * MINUTE_IN_SECONDS );

			return $json_products;

		}
	
	}
	
	/**
	 * Loop products
	 *
	 */
	public function wbte_loop_products( $items, $view ) {

		$wbte_templates = new WbteTemplates();
		
		foreach ( $items as $product ) {
			
			$variations = ( is_a( $product, 'WC_Product' ) && ( $product->is_type( 'variable' ) ) ) ? $product->get_available_variations() : '';
			$variations = ( is_a( $product, 'WC_Product_Subscription' ) && ( $product->is_type( 'variable-subscription' ) ) ) ? $product->get_available_variations() : $variations; 
			
			if ( ! empty( $variations ) ) {
				foreach ( $variations as $variation ) {
					$variation_obj = wc_get_product( $variation['variation_id'] );
					if ( isset( $view ) && 'ext' === $view ) {
						$wbte_templates->extended->wbte_get_table_row( $variation_obj, true, 0 );
					} else {
						$wbte_templates->wbte_get_table_row( $variation_obj, true, 0 );
					}
				}
			} else {
				if ( isset( $view ) && 'ext' === $view ) {
					$wbte_templates->extended->wbte_get_table_row( $product, false, 0 );
				} else {
					$wbte_templates->wbte_get_table_row( $product, false, 0 );
				}
			}
		}

	}

	/**
	 * Get allowed html
	 */
	public function wbte_get_allowed_html() {
		
		$html = array(
			'div'    => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
			'span'   => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
			'i'   => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
			'a'   => array(
				'id'      => array(),
				'class'   => array(),
				'style'   => array(),
				'href'    => array(),
				'onclick' => array(),
			),
			'img'   => array(
				'id'     => array(),
				'class'  => array(),
				'style'  => array(),
				'src'    => array(),
				'height' => array(),
			),
			'select' => array(
				'id'     => array(),
				'class'  => array(),
				'name'   => array(),
				'style'  => array(),
				'src'    => array(),
			),
			'option' => array(
				'id'       => array(),
				'class'    => array(),
				'name'     => array(),
				'style'    => array(),
				'value'    => array(),
				'selected' => array(),
			),
			'optgroup' => array(
				'label' => array(),
			)
		);
		return $html;
	}

	/**
	 * Check for autofocus
	 */
	public function wbte_get_autofocus_value( $type ) {
		
		$options         = get_option( 'wbte_options' );
		$mouseover_value = 'this.focus();';

		if ( 'date' === $type ) {
			if ( 'yes' !== $options['wbte_date_format_autofocus'] ) {
				$mouseover_value = '';
			}
		} else {
			if ( 'yes' === $options['wbte_no_autofocus'] ) {
				$mouseover_value = '';
			}
		}

		return $mouseover_value;
	
	}

	/** 
	* Get options for bulk functions
	*
	* @param var $type type of select.
	* @param var $value value.
	*/
	public function wbte_get_bulk_options( $type, $value = '' ) {

		$option_blank                 = __( 'Select', 'woo-bulk-table-editor' );
		$option_blank_action          = __( 'Select Action', 'woo-bulk-table-editor' );
		$option_blank_bulk            = __( 'Bulk', 'woo-bulk-table-editor' );
		$option_featured              = __( 'Featured', 'woo-bulk-table-editor' );
		$option_increase              = __( 'Increase', 'woo-bulk-table-editor' );
		$option_increase_percent      = __( 'Increase by %', 'woo-bulk-table-editor' );
		$option_increase_number       = __( 'Increase by fixed amount', 'woo-bulk-table-editor' );
		$option_decrease              = __( 'Decrease', 'woo-bulk-table-editor' );
		$option_decrease_percent      = __( 'Decrease by %', 'woo-bulk-table-editor' );
		$option_decrease_number       = __( 'Decrease by fixed amount', 'woo-bulk-table-editor' );
		$option_set_fixed             = __( 'Set fixed', 'woo-bulk-table-editor' );
		$option_set_fixed_number      = __( 'Set fixed amount', 'woo-bulk-table-editor' );
		$option_round_up              = __( 'Round upwards', 'woo-bulk-table-editor' );
		$option_round_down            = __( 'Round downwards', 'woo-bulk-table-editor' );
		$option_round_two             = __( 'Round two decimals', 'woo-bulk-table-editor' );
		$option_yes                   = __( 'Yes', 'woo-bulk-table-editor' );
		$option_no                    = __( 'No', 'woo-bulk-table-editor' );
		$option_yes_bakorder          = __( 'Allow', 'woo-bulk-table-editor' );
		$option_no_backorder          = __( 'Do not allow', 'woo-bulk-table-editor' );
		$option_notify_backorder      = __( 'Allow, but notify customer', 'woo-bulk-table-editor' );
		$option_clear                 = __( 'Clear', 'woo-bulk-table-editor' );
		$option_replace               = __( 'Set SKU', 'woo-bulk-table-editor' );
		$option_set_tags              = __( 'Set tags (use comma to split)', 'woo-bulk-table-editor' );
		$option_add                   = __( 'Add', 'woo-bulk-table-editor' );
		$option_add_after             = __( 'Add after', 'woo-bulk-table-editor' );
		$option_add_before            = __( 'Add before', 'woo-bulk-table-editor' );
		$option_add_id_after          = __( 'Add product ID after', 'woo-bulk-table-editor' );
		$option_add_id_before         = __( 'Add product ID before', 'woo-bulk-table-editor' );
		$option_instock               = __( 'In stock', 'woo-bulk-table-editor' );
		$option_outofstock            = __( 'Out of stock', 'woo-bulk-table-editor' );
		$option_onbackorder           = __( 'On backorder', 'woo-bulk-table-editor' );
		$option_generate_sku          = __( 'Generate SKU', 'woo-bulk-table-editor' );
		$option_upper_case            = __( 'Make uppercase', 'woo-bulk-table-editor' );
		$option_lower_case            = __( 'Make lowercase', 'woo-bulk-table-editor' );
		$option_prev_sale_price       = __( 'Get previous sale prices', 'woo-bulk-table-editor' );
		$option_v_visible             = __( 'Catalog & Search', 'woo-bulk-table-editor' );
		$option_v_catalog             = __( 'Catalog', 'woo-bulk-table-editor' );
		$option_v_search              = __( 'Search', 'woo-bulk-table-editor' );
		$option_v_hidden          	  = __( 'Hidden', 'woo-bulk-table-editor' );
		$option_add_weight       	  = __( 'Add Weight', 'woo-bulk-table-editor' );
		$option_add_length       	  = __( 'Add Length', 'woo-bulk-table-editor' );
		$option_add_width             = __( 'Add Width', 'woo-bulk-table-editor' );
		$option_add_height            = __( 'Add Height', 'woo-bulk-table-editor' );
		$option_increase_self_percent = __( 'Increase Saleprice by %', 'woo-bulk-table-editor' );
		$option_increase_self_number  = __( 'Increase Saleprice by fixed amount', 'woo-bulk-table-editor' );
		$option_decrease_self_number  = __( 'Decrease Saleprice by fixed amount', 'woo-bulk-table-editor' );
		$option_decrease_self_percent = __( 'Decrease Saleprice by %', 'woo-bulk-table-editor' );
		$option_increase_percent_s    = __( 'Increase by % (from Price)', 'woo-bulk-table-editor' );
		$option_increase_number_s     = __( 'Increase by fixed amount (from Price)', 'woo-bulk-table-editor' );
		$option_decrease_percent_s    = __( 'Decrease by % (from Price)', 'woo-bulk-table-editor' );
		$option_decrease_number_s     = __( 'Decrease by fixed amount (from Price)', 'woo-bulk-table-editor' );
		$option_prev_price            = __( 'Get previous price', 'woo-bulk-table-editor' );
		$option_add_brands            = __( 'Add Brands', 'woo-bulk-table-editor' );
		$option_set_brands            = __( 'Set Brands (sets only selected)', 'woo-bulk-table-editor' );
		$option_remove_brands         = __( 'Remove Selected Brands', 'woo-bulk-table-editor' );
		$option_remove                = __( 'Remove', 'woo-bulk-table-editor' );
		$option_set                   = __( 'Set', 'woo-bulk-table-editor' );
		$option_append                = __( 'Append', 'woo-bulk-table-editor' );
		$option_prepend               = __( 'Prepend', 'woo-bulk-table-editor' );
		$option_increment             = __( 'Increment (Number Values)', 'woo-bulk-table-editor' );
		$option_decrement             = __( 'Decrement (Number Values)', 'woo-bulk-table-editor' );
		$option_uppercase             = __( 'Uppercase', 'woo-bulk-table-editor' );
		$option_lowercase             = __( 'Lowercase', 'woo-bulk-table-editor' );
		$option_capitalize            = __( 'Capitalize', 'woo-bulk-table-editor' );
		

		if ( 'stock' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank ); ?></option>
			<option value="up_n"><?php echo esc_html( $option_increase ); ?></option>
			<option value="down_n"><?php echo esc_html( $option_decrease ); ?></option>
			<option value="fix_n"><?php echo esc_html( $option_set_fixed ); ?></option>
			<option value="clear_stock"><?php echo esc_html( $option_clear ); ?></option>
			<?php
		} elseif ( 'featured' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_featured ); ?></option>
			<option value="yes"><?php echo esc_html( $option_yes ); ?></option>
			<option value="no"><?php echo esc_html( $option_no ); ?></option>
			<?php
		} elseif ( 'visibility' === $type ) {
			$selected_txt = 'selected="selected"';
			$default      = $selected_txt;
			$opt_visible  = '';
			$opt_catalog  = '';
			$opt_search   = '';
			$opt_hidden   = '';
			if ( ! empty( $value ) ) {
				switch ( $value ) {
					case 'visible':
						$opt_visible = $selected_txt;
						$default     = '';
						break;
					case 'catalog':
						$opt_catalog = $selected_txt;
						$default     = '';
						break;
					case 'search':
						$opt_search = $selected_txt;
						$default    = '';
						break;
					case 'hidden':
						$opt_hidden = $selected_txt;
						$default    = '';
						break;
				}
			}
			?>
			<option value="visible" <?php echo esc_attr( $default ); ?>><?php echo esc_html( $option_blank ); ?></option>
			<option value="visible" <?php echo esc_attr( $opt_visible ); ?>><?php echo esc_html( $option_v_visible ); ?></option>
			<option value="catalog" <?php echo esc_attr( $opt_catalog ); ?>><?php echo esc_html( $option_v_catalog ); ?></option>
			<option value="search" <?php echo esc_attr( $opt_search ); ?>><?php echo esc_html( $option_v_search ); ?></option>
			<option value="hidden" <?php echo esc_attr( $opt_hidden ); ?>><?php echo esc_html( $option_v_hidden ); ?></option>
			<?php
		} elseif ( 'backorder' === $type ) {
			$selected_txt = 'selected="selected"';
			$default      = $selected_txt;
			$opt_no       = '';
			$opt_notify   = '';
			$opt_yes      = '';
			if ( ! empty( $value ) ) {
				switch ( $value ) {
					case 'no':
						$opt_no  = $selected_txt;
						$default = '';
						break;
					case 'notify':
						$opt_notify = $selected_txt;
						$default    = '';
						break;
					case 'yes':
						$opt_yes = $selected_txt;
						$default = '';
						break;
				}
			}
			?>
			<option value="0" <?php echo esc_attr( $default ); ?>><?php echo esc_html( $option_blank ); ?></option>
			<option value="no" <?php echo esc_attr( $opt_no ); ?>><?php echo esc_html( $option_no_backorder ); ?></option>
			<option value="notify" <?php echo esc_attr( $opt_notify ); ?>><?php echo esc_html( $option_notify_backorder ); ?></option>
			<option value="yes" <?php echo esc_attr( $opt_yes ); ?>><?php echo esc_html( $option_yes_bakorder ); ?></option>
			<?php
		} elseif ( 'sku' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank ); ?></option>
			<option value="replace"><?php echo esc_html( $option_replace ); ?></option>
			<option value="generate"><?php echo esc_html( $option_generate_sku ); ?></option>
			<option value="add-before"><?php echo esc_html( $option_add_before ); ?></option>
			<option value="add-after"><?php echo esc_html( $option_add_after ); ?></option>
			<option value="add-id-before"><?php echo esc_html( $option_add_id_before ); ?></option>
			<option value="add-id-after"><?php echo esc_html( $option_add_id_after ); ?></option>
			<option value="add-weight"><?php echo esc_html( $option_add_weight ); ?></option>
			<option value="add-length"><?php echo esc_html( $option_add_length); ?></option>
			<option value="add-width"><?php echo esc_html( $option_add_width ); ?></option>
			<option value="add-height"><?php echo esc_html( $option_add_height ); ?></option>
			<option value="toupper"><?php echo esc_html( $option_upper_case ); ?></option>
			<option value="tolower"><?php echo esc_html( $option_lower_case ); ?></option>
			<option value="clear"><?php echo esc_html( $option_clear ); ?></option>
			<?php
		} elseif ( 'tags' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank ); ?></option>
			<option value="add"><?php echo esc_html( $option_add ); ?></option>
			<option value="replace"><?php echo esc_html( $option_set_tags ); ?></option>
			<option value="toupper"><?php echo esc_html( $option_upper_case ); ?></option>
			<option value="tolower"><?php echo esc_html( $option_lower_case ); ?></option>
			<option value="clear"><?php echo esc_html( $option_clear ); ?></option>
			<?php
		} elseif ( 'brand' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank ); ?></option>
			<option value="add"><?php echo esc_html( $option_add_brands ); ?></option>
			<option value="set"><?php echo esc_html( $option_set_brands ); ?></option>
			<option value="remove"><?php echo esc_html( $option_remove_brands ); ?></option>
			<option value="clear"><?php echo esc_html( $option_clear ); ?></option>
			<?php	
		} elseif ( 'bulkcat' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank_bulk ); ?></option>
			<option value="add"><?php echo esc_html( $option_add ); ?></option>
			<option value="set"><?php echo esc_html( $option_set ); ?></option>
			<option value="remove"><?php echo esc_html( $option_remove ); ?></option>
			<option value="clear"><?php echo esc_html( $option_clear ); ?></option>
			<?php
		} elseif ( 'bulkcat2' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank ); ?></option>
			<option value="add"><?php echo esc_html( $option_add ); ?></option>
			<option value="set"><?php echo esc_html( $option_set ); ?></option>
			<option value="remove"><?php echo esc_html( $option_remove ); ?></option>
			<option value="clear"><?php echo esc_html( $option_clear ); ?></option>
			<?php	
		} elseif ( 'customs' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank_action ); ?></option>
			<option value="add"><?php echo esc_html( $option_add ); ?></option>
			<option value="set"><?php echo esc_html( $option_set ); ?></option>
			<option value="remove"><?php echo esc_html( $option_remove ); ?></option>
			<option value="clear"><?php echo esc_html( $option_clear ); ?></option>
			<option value="append"><?php echo esc_html( $option_append ); ?></option>
			<option value="prepend"><?php echo esc_html( $option_prepend ); ?></option>
			<option value="increment"><?php echo esc_html( $option_increment ); ?></option>
			<option value="decrement"><?php echo esc_html( $option_decrement ); ?></option>
			<option value="uppercase"><?php echo esc_html( $option_uppercase ); ?></option>
			<option value="lowercase"><?php echo esc_html( $option_lowercase ); ?></option>
			<option value="capitalize"><?php echo esc_html( $option_capitalize ); ?></option>
			<?php		
		} elseif ( 'instock' === $type ) {
			$selected_txt = 'selected="selected"';
			$default      = $selected_txt;
			$opt_in       = '';
			$opt_out_of   = '';
			$opt_on_back  = '';
			if ( ! empty( $value ) ) {
				switch ( $value ) {
					case 'instock':
						$opt_in  = $selected_txt;
						$default = '';
						break;
					case 'outofstock':
						$opt_out_of = $selected_txt;
						$default    = '';
						break;
					case 'onbackorder':
						$opt_on_back = $selected_txt;
						$default     = '';
						break;
				}
			}
			?>
			<option value="0" <?php echo esc_attr( $default ); ?>><?php echo esc_html( $option_blank ); ?></option>
			<option value="instock" <?php echo esc_attr( $opt_in ); ?>><?php echo esc_html( $option_instock ); ?></option>
			<option value="outofstock" <?php echo esc_attr( $opt_out_of ); ?>><?php echo esc_html( $option_outofstock ); ?></option>
			<option value="onbackorder" <?php echo esc_attr( $opt_on_back ); ?>><?php echo esc_html( $option_onbackorder ); ?></option>
			<?php
		} elseif ( 'numbers' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank ); ?></option>
			<option value="fix_n"><?php echo esc_html( $option_set_fixed ); ?></option>
			<option value="up_n"><?php echo esc_html( $option_increase ); ?></option>
			<option value="down_n"><?php echo esc_html( $option_decrease ); ?></option>
			<option value="clear"><?php echo esc_html( $option_clear ); ?></option>
			<?php
		} elseif ( 'sale_price' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank ); ?></option>
			<option value="up_p"><?php echo esc_html( $option_increase_percent_s ); ?></option>
			<option value="up_n"><?php echo esc_html( $option_increase_number_s ); ?></option>
			<option value="up_s_p"><?php echo esc_html( $option_increase_self_percent ); ?></option>
			<option value="up_s_n"><?php echo esc_html( $option_increase_self_number ); ?></option>
			<option value="down_p"><?php echo esc_html( $option_decrease_percent_s ); ?></option>
			<option value="down_n"><?php echo esc_html( $option_decrease_number_s ); ?></option>
			<option value="down_s_p"><?php echo esc_html( $option_decrease_self_percent ); ?></option>
			<option value="down_s_n"><?php echo esc_html( $option_decrease_self_number ); ?></option>
			<option value="fix_n"><?php echo esc_html( $option_set_fixed_number ); ?></option>
			<option value="round_up"><?php echo esc_html( $option_round_up ); ?></option>
			<option value="round_down"><?php echo esc_html( $option_round_down ); ?></option>
			<option value="round_two"><?php echo esc_html( $option_round_two ); ?></option>
			<option value="prev_sale_price"><?php echo esc_html( $option_prev_sale_price ); ?></option>
			<?php
		} elseif ( 'price' === $type ) {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank ); ?></option>
			<option value="up_p"><?php echo esc_html( $option_increase_percent ); ?></option>
			<option value="up_n"><?php echo esc_html( $option_increase_number ); ?></option>
			<option value="down_p"><?php echo esc_html( $option_decrease_percent ); ?></option>
			<option value="down_n"><?php echo esc_html( $option_decrease_number ); ?></option>
			<option value="fix_n"><?php echo esc_html( $option_set_fixed_number ); ?></option>
			<option value="round_up"><?php echo esc_html( $option_round_up ); ?></option>
			<option value="round_down"><?php echo esc_html( $option_round_down ); ?></option>
			<option value="round_two"><?php echo esc_html( $option_round_two ); ?></option>
			<option value="prev_price"><?php echo esc_html( $option_prev_price ); ?></option>
			<option value="clear_price"><?php echo esc_html( $option_clear ); ?></option>
			<?php
		} elseif ( 'shipping' === $type ) {
			$shipping_classes = get_terms( array( 'taxonomy' => 'product_shipping_class', 'hide_empty' => false ) );
			?>
			<option value="0" <?php echo esc_attr( '' === $value ? 'selected="selected"' : '' ); ?>><?php echo esc_html( $option_blank ); ?></option>
			<?php
			foreach ( $shipping_classes as $shipping ) {
				?>
				<option value="<?php echo esc_attr( $shipping->term_id ); ?>" <?php echo esc_attr( $shipping->term_id === $value ? 'selected="selected"' : '' ); ?>><?php echo esc_attr( $shipping->name ); ?></option>
				<?php
			}
		} else {
			?>
			<option value="0" selected="selected"><?php echo esc_html( $option_blank ); ?></option>
			<option value="up_p"><?php echo esc_html( $option_increase_percent ); ?></option>
			<option value="up_n"><?php echo esc_html( $option_increase_number ); ?></option>
			<option value="down_p"><?php echo esc_html( $option_decrease_percent ); ?></option>
			<option value="down_n"><?php echo esc_html( $option_decrease_number ); ?></option>
			<option value="fix_n"><?php echo esc_html( $option_set_fixed_number ); ?></option>
			<option value="round_up"><?php echo esc_html( $option_round_up ); ?></option>
			<option value="round_down"><?php echo esc_html( $option_round_down ); ?></option>
			<option value="round_two"><?php echo esc_html( $option_round_two ); ?></option>
			<?php
		}

	}

	/**
	 * Get selected vendors
	 *
	 * @param var $product_id product id.
	 */
	public function wbte_get_vendor_selected( $product_id ) {
		
		if ( empty( WC_PRODUCT_VENDORS_TAXONOMY ) ) {
			return;
		}

		$args = array(
			'hide_empty'   => false,
			'hierarchical' => false,
			'taxonomy'     => WC_PRODUCT_VENDORS_TAXONOMY,
		);

		$terms = get_terms( $args );

		if ( ! empty( $terms ) ) {
			
			$post_term = wp_get_post_terms( $product_id, WC_PRODUCT_VENDORS_TAXONOMY );
			$post_term = ! empty( $post_term ) ? $post_term[0]->term_id : '';
			$output    = '<select class="input-select" name="vendor_selected">';
			$output   .= '<option value="">' . esc_html__( 'Select', 'woo-bulk-table-editor' ) . '</option>';

			foreach ( $terms as $term ) {
				$output .= '<option value="' . esc_attr( $term->term_id ) . '" ' . selected( $post_term, $term->term_id, false ) . '>' . esc_html( $term->name ) . '</option>';
			}

			$output .= '</select>';

			echo wp_kses( $output, $this->wbte_get_allowed_html() );
		} 
	}

	/**
	 * Get vendors
	 *
	 */
	public function wbte_get_vendor_select() {
		
		$args = array(
			'hide_empty'   => false,
			'hierarchical' => false,
			'taxonomy'     => WC_PRODUCT_VENDORS_TAXONOMY,
		);

		$terms = get_terms( $args );

		if ( ! empty( $terms ) ) {

			$output = '<option value="">' . esc_html__( 'Select', 'woo-bulk-table-editor' ) . '</option>';

			foreach ( $terms as $term ) {
				$output .= '<option value="' . esc_attr( $term->term_id ) . '">' . esc_html( $term->name ) . '</option>';
			}

			echo wp_kses( $output, $this->wbte_get_allowed_html() );

		} else {
			esc_attr_e( 'None found', 'woo-bulk-table-editor' ); 
		}
	}

	/**
	 * Get brands (slugs) on a product
	 */
	public function wbte_get_product_brands( $product_id, $type = 'slug' ) {
		
		$list = '';

		if ( empty( $product_id ) ) {
			return $list;
		}
		
		if ( class_exists( 'WC_Brands' ) ) {
			$brands = wp_get_post_terms( $product_id, 'product_brand' );
			if ( ! empty( $brands ) ) {
				foreach ( $brands as $brand ) {
					if ( 'name' === $type ) {
						$list .= $brand->name . ',';
					} else {
						$list .= $brand->slug . ',';
					}
				}
				$list = rtrim( $list, ',' );
			}
		}

		return $list;
	}

	/**
	 * Create export file (.csv)
	 */
	public function wbte_create_csv_export_file() {
		
		$file_name    = 'wbte-products-' . gmdate('Y-M-d H:i') . '.csv';
		$p_obj        = $this->wbte_get_json_products();
		$products     = (array) $p_obj['rows'][0];
		$custom_price = $this->wbte_get_custom_price_info();
		
		$head_csv = array(
			__( 'ID', 'woo-bulk-table-editor' ),
			__( 'Name', 'woo-bulk-table-editor' ),
			__( 'SKU', 'woo-bulk-table-editor' ),
			__( 'Categories', 'woo-bulk-table-editor' ),
			__( 'Featured', 'woo-bulk-table-editor' ),
			__( 'Price', 'woo-bulk-table-editor' ), 
			__( 'Sale price', 'woo-bulk-table-editor' ),
			__( 'Sale start date', 'woo-bulk-table-editor' ),
			__( 'Sale end date', 'woo-bulk-table-editor' ),
			__( 'Sale %', 'woo-bulk-table-editor' ),
			__( 'Stock', 'woo-bulk-table-editor' ),
			__( 'Stock value', 'woo-bulk-table-editor' ),
			__( 'Tags', 'woo-bulk-table-editor' ),
		);

		if ( 'yes' === $custom_price['active'] ) {
			array_splice( $head_csv, 5, 0, $custom_price['name'] );
		}

		$rows = array();
		
		foreach ( $products as $key => $p ) {
				
			$id = ( ! is_null( $p ) && $p->get_id() ) ? $p->get_id() : false;
			
			if ( ! $id ) {
				continue;
			}

			$variations = ( is_a( $p, 'WC_Product' ) && ( $p->is_type( 'variable' ) ) ) ? $p->get_available_variations() : '';
			$variations = ( is_a( $p, 'WC_Product_Subscription' ) && ( $p->is_type( 'variable-subscription' ) ) ) ? $p->get_available_variations() : $variations; 

			if ( ! empty( $variations ) ) {
				foreach ( $variations as $variation ) {
					$variation_obj = wc_get_product( $variation['variation_id'] );
					array_push( $rows, $this->wbte_create_csv_row( $variation_obj ) );
				}
			} else {
				array_push( $rows, $this->wbte_create_csv_row( $p ) );
			}

		}
	  
		header( 'Content-Type: text/csv' ); 
		header( 'Content-Disposition: attachment; filename="' . $file_name . '"' );
		header( 'Pragma: no-cache' ); 
		header( 'Expires: Sat, 26 Jul 2010 05:00:00 GMT' );

		$output = tmpfile();

		// Put the headers
		fputcsv( $output, $head_csv, ';' );

		// Put the rows
		foreach ( $rows as $row ) {
			fputcsv( $output, (array) $row, ';' );
		}
	
		rewind( $output ); 
		readfile( stream_get_meta_data( $output )['uri'] );

		exit;

	}

	/**
	 * Create the export row
	 */
	private function wbte_create_csv_row( $p ) {

		$t_sep        = get_option( 'woocommerce_price_thousand_sep' );
		$d_point      = get_option( 'woocommerce_price_decimal_sep' );
		$decimals     = get_option( 'woocommerce_price_num_decimals' );
		$custom_price = $this->wbte_get_custom_price_info();
		$product_name = $p->get_title();
		$attributes   = $p->get_attributes();
		$date_format  = get_option( 'date_format' );	

		if ( isset( $attributes ) && is_array( $attributes ) ) {
			
			foreach ( $attributes as $key => $val ) {
				if ( isset( $val ) && is_string( $val ) && strlen( $val ) > 0 ) {
					$product_name .= ', ' . $val;
				}
			}

			$id = $p->get_parent_id();
		
			if ( 0 === $id ) {
				$id = $p->get_id();
			}
		}

		$terms        = get_the_terms( $id, 'product_cat' );
		$categories   = '';
		$tags_arr     = get_the_terms( $id, 'product_tag' );
		$tags         = '';
		$sale_percent = '';

		if ( $terms ) {
			foreach ( $terms as $category ) {
				$categories .= $category->name . ', ';
			}
			$categories = substr( $categories, 0, strlen( $categories ) -2 );
		}
		
	
		if ( $tags_arr ) {
			foreach ( $tags_arr as $obj ) {
				$tags .= $obj->name . ', ';
			}
			$tags = substr( $tags, 0, strlen( $tags ) - 2 );
		}

		if ( strlen( $p->get_sale_price() ) > 0 ) {
			$sale_percent = ( $p->get_regular_price() - $p->get_sale_price() ) * 100 / $p->get_regular_price();
		}

		$row = array(
			$p->get_id(),
			$product_name,
			$p->get_sku(),
			$categories,
			( $p->get_featured() ) ? 'on' : '',
			( $p->get_regular_price() ) ? $p->get_regular_price() : '',
			( $p->get_sale_price() ) ? $p->get_sale_price() : '',
			( $p->get_date_on_sale_from() ) ? date_format( $p->get_date_on_sale_from(), $date_format ) : '',
			( $p->get_date_on_sale_to() ) ? date_format( $p->get_date_on_sale_to(), $date_format ) : '',
			( '' !== $sale_percent ) ? number_format( (float) $sale_percent, $decimals, $d_point, $t_sep ) : '',//sale %
			$p->get_stock_quantity(),
			( $p->get_stock_quantity() > 0 && $p->get_regular_price() > 0 ) ? number_format( (float) ( (int) $p->get_stock_quantity() * $p->get_regular_price() ), $decimals, $d_point, $t_sep ) : '',
			$tags,
		);

		if ( 'yes' === $custom_price['active'] ) {
			array_splice( $row, 5, 0, get_post_meta( $id, $custom_price['price'], true ) );
		}

		return $row;
	}

	/**
	 * Set upsell and cross sells on products
	 */
	public static function wbte_upsell_set_linked_products_callback( $data ) {
		
		check_ajax_referer( 'wbte_linked_id', 'nonce' );
		
		$data     = wp_unslash( $_POST );
		$type     = $data['linktype'];
		$products = $data['products'];
		$upsells  = $data['upsells'];

		foreach ( $products as $productId ) {

			$product = wc_get_product( (int) $productId );
			$ups     = $product->get_upsell_ids();
			$cross   = $product->get_cross_sell_ids();

			switch ( $type ) {
				case 'up':
					$product->set_upsell_ids( array_unique( array_merge( $ups, (array) $upsells ), SORT_NUMERIC ) );
					break;
				case 'cross':
					$product->set_cross_sell_ids( array_unique( array_merge( $cross, (array) $upsells ), SORT_NUMERIC ) );
					break;
				case 'up-cross':
					$product->set_upsell_ids( array_unique( array_merge( $ups, (array) $upsells ), SORT_NUMERIC ) );
					$product->set_cross_sell_ids( array_unique( array_merge( $cross, (array) $upsells ), SORT_NUMERIC ) );
					break;
			}

			$product->save();
		}

		die();
	}

	/**
	 * Remove linked product (upsell or cross-sell)
	 */
	public static function wbte_upsell_remove_linked_product_callback( $data ) {
		
		check_ajax_referer( 'wbte_linked_id', 'nonce' );
		
		$data     = wp_unslash( $_POST );
		$type     = $data['linktype'];
		$product  = $data['product'];
		$upsellId = $data['upsell'];

		$product = wc_get_product( (int) $product );
		$ups     = $product->get_upsell_ids();
		$cross   = $product->get_cross_sell_ids();

		if ( 'up' === $type ) {
			$key = array_search( $upsellId, $ups );
			if ( false !== $key ) {
				unset( $ups[$key] );
			}
			$product->set_upsell_ids( $ups );
		} else {
			$key = array_search( $upsellId, $cross );
			if ( false !== $key ) {
				unset( $cross[$key] );
			}
			$product->set_cross_sell_ids( $cross );
		}

		$product->save();

		die();
	}

}

